#include <QSettings>
#include <QCoreApplication>
#include <QFile>
#include <QDir>
#include <QMutex>
#include <iostream>
#include <gsl/gsl_errno.h>
#include "core.h"

namespace smile{

CSmileCore* core;   // the instance of core object holding all non-gui data
CConfigData config;

// gsl error handling
static void gsl_verbose_error_handler (const char *reason, const char *file, int line, int gsl_errno)
{
    if(gsl_errno == GSL_ETOL) return;  // do nothing
    char buffer[1024];
    sprintf(buffer, "GSL error %d in %s at line %d: %s\n", gsl_errno, file, line, reason);
    std::cerr << buffer;
}

CSmileCore::CSmileCore()
{
    core = this;     // create the global pointer
    WorkDir = QDir::currentPath(); 
    AppDir  = getDirName(QCoreApplication::applicationFilePath());
    TempDir = "";
    maxNumThread=std::max(1, QThread::idealThreadCount());
    ConsoleMode=false;
    potential=NULL;
    orbits=NULL;
    model=NULL;
    orbit=NULL;
    CalcThr=NULL;
    SchwThr=NULL;
    N_dim=3; q=p=1; Mbh=0; Rc=1; Gamma=1; Alpha=1;
    Ncoefs_radial=10; Ncoefs_angular=4;
    PotentialType = CPotential::PT_LOG;
    gsl_set_error_handler(&gsl_verbose_error_handler);
}

void CSmileCore::initPotential(QString potentialName)
{
    if(potential) 
        delete potential;
    if(potentialName==CPotentialLog::myName())
        potential=new CPotentialLog(N_dim, q, p, Mbh, Rc);
    if(potentialName==CPotentialHarmonic::myName())
        potential=new CPotentialHarmonic(N_dim, q, p, Mbh);
    else if(potentialName==CPotentialDehnen::myName())
        potential=new CPotentialDehnen(N_dim, q, p, Mbh, Gamma);
    else if(potentialName==CPotentialDehnenBSE::myName())
        potential=new CPotentialDehnenBSE(N_dim, q, p, Mbh, Gamma, Ncoefs_radial, Ncoefs_angular);
    else if(potentialName==CPotentialScaleFree::myName())
        potential=new CPotentialScaleFree(N_dim, q, p, Mbh, Gamma);
    else if(potentialName==CPotentialScaleFreeBSE::myName())
        potential=new CPotentialScaleFreeBSE(N_dim, q, p, Mbh, Gamma, Ncoefs_angular);
    else if(potentialName==CPotentialPlummerBSE::myName())
        potential=new CPotentialPlummerBSE(N_dim, q, p, Mbh, Ncoefs_radial, Ncoefs_angular);
    else if(potentialName==CPotentialBSE::myName() || potentialName==CPotentialNB::myName())
        initPotentialNB(potentialName==CPotentialNB::myName());
    else
        potential=new CPotentialLog(N_dim, q, p, Mbh, Rc);
    PotentialType = potential->PotentialType();
}
void CSmileCore::initPotential(CPotential::POTENTIALTYPE potentialType)
{
    if(potential) 
        delete potential;
    if(potentialType==CPotential::PT_LOG)
        potential=new CPotentialLog(N_dim, q, p, Mbh, Rc);
    if(potentialType==CPotential::PT_HARMONIC)
        potential=new CPotentialHarmonic(N_dim, q, p, Mbh);
    else if(potentialType==CPotential::PT_DEHNEN)
        potential=new CPotentialDehnen(N_dim, q, p, Mbh, Gamma);
    else if(potentialType==CPotential::PT_DEHNENBSE)
        potential=new CPotentialDehnenBSE(N_dim, q, p, Mbh, Gamma, Ncoefs_radial, Ncoefs_angular);
    else if(potentialType==CPotential::PT_SCALEFREE)
        potential=new CPotentialScaleFree(N_dim, q, p, Mbh, Gamma);
    else if(potentialType==CPotential::PT_SCALEFREEBSE)
        potential=new CPotentialScaleFreeBSE(N_dim, q, p, Mbh, Gamma, Ncoefs_angular);
    else if(potentialType==CPotential::PT_BSE || potentialType==CPotential::PT_NB)
        initPotentialNB(potentialType==CPotential::PT_NB);
    else if(potentialType==CPotential::PT_PLUMMERBSE)
        potential=new CPotentialPlummerBSE(N_dim, q, p, Mbh, Ncoefs_radial, Ncoefs_angular);
    else 
        potential=new CPotentialLog(N_dim, q, p, Mbh, Rc);
    PotentialType = potential->PotentialType();
}

void CSmileCore::initPotentialNB(bool frozenN)
{
    N_dim=3;
    vectord X, Y, Z, mass;
    QFileInfo NbodyFileName(QDir(WorkDir), NbodyFile);
    QFile file(NbodyFileName.absoluteFilePath());
    if (!file.open(QFile::ReadOnly | QFile::Text)) 
    {
        if(frozenN) potential=new CPotentialNB(N_dim, q, p, Mbh, Gamma, 0, 0); else potential=new CPotentialBSE(N_dim, q, p, Mbh, Gamma, Alpha, 1, 1, 0);
        return;
    }
    QTextStream strm(&file);
    while(!strm.atEnd())
    {
        QString ln = strm.readLine();
        QStringList fields=ln.split(QRegExp("[#\\s]+"));
        if(fields.size()>=7)
        {
            bool ok;
            fields[0].toDouble(&ok);
            if(!ok) continue;
            X.push_back(fields[0].toDouble());
            Y.push_back(fields[1].toDouble());
            Z.push_back(fields[2].toDouble());
            mass.push_back(fields[6].toDouble());
        }
    }
    if(X.size()<=0)
    {
        if(frozenN) potential=new CPotentialNB(N_dim, q, p, Mbh, Gamma, 0, 0); 
               else potential=new CPotentialBSE(N_dim, q, p, Mbh, Gamma, Alpha, 1, 1, 0);
        return;
    }
    if(frozenN)
        potential = new CPotentialNB(N_dim, q, p, Mbh, Gamma, config.nbpotEps, config.nbpotTol, X.size(), &(X.front()), &(Y.front()), &(Z.front()), &(mass.front()));
    else
    {
        potential = new CPotentialBSE(N_dim, q, p, Mbh, Gamma, Alpha, Ncoefs_radial, Ncoefs_angular, X.size(), &(X.front()), &(Y.front()), &(Z.front()), &(mass.front()));
    }
}

//// ----------- orbit integration ------------ ////
bool CSmileCore::initIC()
{
    bool changed=false;
    if(useICe)
    {
        if((PotentialType==CPotential::PT_DEHNEN || PotentialType==CPotential::PT_DEHNENBSE) && (E>-1e-5))
            E=-1e-5;  // for Dehnen model E<0
        icX=potential->longaxisradius(E);
        icY=icZ=0;
        icVx=icVy=icVz=0;
    }
    else
    {
        double V2 = sqr(icVx) + sqr(icVy) + sqr(icVz);
        E = potential->Phi(icX, icY, icZ) + V2/2;
        if((PotentialType==CPotential::PT_DEHNEN || PotentialType==CPotential::PT_DEHNENBSE) && (E>-1e-5))
        {   // for Dehnen model E<0, so if it exceeds this, reset V to zero
            E=potential->Phi(icX, icY, icZ);
            icVx=icVy=icVz=0;
            changed=true;
        }
    }
    // find maximal x-axis elongation for given energy
    Xmax = potential->longaxisradius(E);
    // calculate T_orb (for x-axis orbit)
    Torbx= potential->longaxisperiod(E, Xmax);
    return changed;
}

void CSmileCore::startOrbit()
{
    if(Torbx<0) return;  // error
    if(CalcThr!=NULL)
    {
        CalcThr->deleteLater();
    }
    double IP[N_DIM], IV[N_DIM];
    IP[0]=icX;  IP[1]=icY;  IP[2]=icZ;
    IV[0]=icVx; IV[1]=icVy; IV[2]=icVz;
    //initIC();
    if(orbit!=NULL)
    {
        delete orbit->potential;  // manually delete, since it was created by cloning externally to the object
        delete orbit;
    }
    orbit = new COrbit(potential->clone(), NULL, IP, IV, Torbx/config.intTimeStepsPerPeriod, Torbx, config.calcLyapunov, config.usePS);
    CalcThr = new CCalcThread(orbit, config.intTimeInPeriods*Torbx);
    connect(CalcThr, SIGNAL(calcFinished()), SLOT(coreOrbitFinished()));
    CalcThr->start();
}

void CSmileCore::coreOrbitFinished()
{
    // store orbital points
    int nPoints = orbit->nActual;
    // calc maxdist
    maxdist=0;
    maxr=orbit->maxr;
    for(int i=1; i<nPoints; i++)
    {
        double dist = sqr(orbit->tp[0][i]-orbit->tp[0][i-1]) + sqr(orbit->tp[1][i]-orbit->tp[1][i-1]) + ((potential->N_dim==3)?sqr(orbit->tp[2][i]-orbit->tp[2][i-1]):0);
        if(dist>maxdist)
            maxdist=dist;
    }
    maxdist *= 1.0000001;

    emit signalOrbitFinished();   // in GUI - displays various information, called immediately in the same thread before orbit is destroyed

    CalcThr->deleteLater();
    CalcThr=NULL;
}

bool CSmileCore::exportOrbit(QString fileName)
{
    if(orbit==NULL || orbit->tp[0].size()==0) return false;
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) 
        return false;
    QTextStream out(&file);
	out.setRealNumberPrecision(8);
    out << "t\tx\ty\tz\tvx\tvy\tvz\tE\n";
    int npoints=orbit->nActual;
    for(int i=0; i<npoints; i++)
    {
        out << (i*orbit->timeStep) << "\t" <<
            orbit->tp[0][i] << "\t" << orbit->tp[1][i] << "\t" << ((N_dim==3)?orbit->tp[2][i]:0) << "\t" << 
            orbit->tv[0][i] << "\t" << orbit->tv[1][i] << "\t" << ((N_dim==3)?orbit->tv[2][i]:0) << "\t" <<
            "\n";
    }
    return true;
}

bool CSmileCore::exportPotential(QString fileName)
{
    if(potential==NULL) return false;
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) 
        return false;
    QTextStream out(&file);
	out.setRealNumberPrecision(8);
    out << "x\ty\tz\tPhi\tFx\tFy\tFz\tRho\n";
    // export potential and its derivatives in a set of points: along three principal axes, three diagonals in each plane, and a x=y=z line
    const int Npoints=100, Ndet=10, rmax=100;
    double axiscoefs[7][3] = { {1,0,0}, {0,1,0}, {0,0,1}, {0.7,0.7,0}, {0,0.7,0.7}, {0.7,0,0.7}, {0.58,0.58,0.58} };
    for(int d=0; d<7; d++)
    {
        for(int i=1; i<=Npoints; i++)
        {
            double r=(exp((i-Npoints)*1.0/Ndet)-exp(-Npoints*1.0/Ndet))/(1-exp(-Npoints*1.0/Ndet))*rmax;
            double v[6]={r*axiscoefs[d][0], r*axiscoefs[d][1], r*axiscoefs[d][2], 0, 0, 0};
            double f[8];
            double Phi = potential->Phi(v[0],v[1],v[2]);
            potential->DiffEq(N_DIM*2, 0, v, f);
            out << v[0] << "\t" << v[1] << "\t" << v[2] << "\t" << Phi << "\t" 
                << -f[3] << "\t" << -f[4] << "\t" << -f[5] << "\t" << potential->Rho(v[0],v[1],v[2]) << "\n";
        }
    }
    return true;
}

//// ----------- Frequency map / Schwarzschild modelling ------------- ////

void CSmileCore::StartOrbitLibrary(const char* finishSignal)
{
    for(int t=0; t<maxNumThread; t++)
    {
        CalcManyThr.push_back(new CCalcManyThread(orbits));
        connect(CalcManyThr[t], SIGNAL(calcFinished()), finishSignal);
        CalcManyThr[t]->start(QThread::LowPriority);
    }
    emit signalInfo("Starting integration..");
    myTimer = startTimer(2000);  // to refresh progress
}

void CSmileCore::BuildFreqMapIC(double energy, bool useExistingIC)
{
    if(!potential) return;
    if(useExistingIC && orbits!=NULL && orbits->OrbitList.size()>0)
        orbits->modifyInitialConditions(potential);
    else
    {
        if(orbits)
            delete orbits;
        orbits=new COrbitLibrary(potential, NULL);
        emit signalInfo("Preparing initial conditions..");
        orbits->prepareInitialConditions(config.fm_numOrbitsStationary, config.fm_numOrbitsPrincipalPlane, config.fm_numOrbitsYalpha, config.fm_numOrbitsRandom, energy);
    }
}

void CSmileCore::coreFreqMapFinished()
{
    CalcManyThr.clear();
    killTimer(myTimer);
    myTimer=0;
    // delete non-used orbits (if integration threads were terminated prematurely, there exist some)
    orbits->removeUnused();
    orbits->markChaoticOrbits();
    // finally, send signal to the parent
    emit signalFreqMapFinished();
}

void CSmileCore::BuildSchwIC(bool useExistingIC)
{
    if(!potential || potential->N_dim!=3) return;
    // Prepare initial conditions
    if(model)
        delete model;
    emit signalInfo("Preparing model..");
    model=new CSchwarzschildModel(potential, config.sm_numShells, config.sm_linesPerSegment);
    if(useExistingIC && orbits!=NULL && orbits->OrbitList.size()>0)
    {
        orbits->modifyInitialConditions(potential, model->grid);
    }
    else
    {
        useExistingIC=false;
        if(orbits)
            delete orbits;
        orbits=new COrbitLibrary(potential, model->grid);
    }
    model->orbits=orbits; 
    if(!useExistingIC)
    {
        emit signalInfo("Preparing initial conditions..");
        orbits->prepareInitialConditions(config.sm_numOrbitsStationary, config.sm_numOrbitsPrincipalPlane, config.sm_numOrbitsYalpha, config.sm_numOrbitsRandom, model);
    }
}

void CSmileCore::coreSchwOrbitLibraryFinished()
{
    CalcManyThr.clear();
    killTimer(myTimer);
    myTimer=0;
    // delete non-used orbits (if integration threads were terminated prematurely, there exist some)
    orbits->removeUnused();
    orbits->markChaoticOrbits();
    // finally, send signal to the parent
    emit signalSchwOrbitLibraryFinished();
}

void CSmileCore::SchwStartOptimization(CSchwarzschildThread::OPERATION operation)
{
    if(SchwThr!=NULL || model==NULL || operation==CSchwarzschildThread::SM_EXPORTNBODY)
        return;      // do not allow to run two threads at once
    SchwThr = new CSchwarzschildThread(model, operation);
    connect(SchwThr, SIGNAL(calcFinished(double)), SLOT(coreSchwOptimizationFinished(double)));
    connect(SchwThr, SIGNAL(calcProgress(const QString&)), SIGNAL(signalInfo(const QString&)));
    SchwThr->start();
}

void CSmileCore::SchwExportNbody(int numPoints, QString fileName, int refineFactor)
{
    if(SchwThr!=NULL || model==NULL)
        return;
    int numReintegrate = model->exportNbodyPrepare(numPoints, fileName, refineFactor);
    if(numReintegrate)
    {   // requires reintegration
        StartOrbitLibrary(SLOT(coreSchwExportNbody()));
        emit signalInfo("Reintegrating "+QString::number(numReintegrate)+" orbits..");
    }
    else
        emit coreSchwExportNbody();
}

void CSmileCore::coreSchwExportNbody()
{
    emit signalInfo("Exporting Nbody model..");
    if(CalcManyThr.size()>0)
        CalcManyThr.clear();  // clean up threads used to reintegrate
    if(SchwThr!=NULL || model==NULL)
        return;      // do not allow to run two threads at once
    SchwThr = new CSchwarzschildThread(model, CSchwarzschildThread::SM_EXPORTNBODY);
    connect(SchwThr, SIGNAL(calcFinished(double)), SLOT(coreSchwOptimizationFinished(double)));
    connect(SchwThr, SIGNAL(calcProgress(const QString&)), SIGNAL(signalInfo(const QString&)));
    SchwThr->start();
}

void CSmileCore::coreSchwOptimizationFinished(double result)
{
    if(result>=0) emit signalSchwOptimizationFinished(
        (SchwThr->nIter>=0 ? QString::number(SchwThr->nIter) + " iterations\n" : "") +
        QString::number(model->nOrbitsNonZero)+" orbits out of "+QString::number(orbits->nComplete)+" used\n" +
        QString::number(model->nCellsInfeasible)+" cells out of "+QString::number(model->grid->cellsInModel)+" infeasible\n" +
        QString::number(result)+" total difference\n" +
        QString::number(model->sumSquaredWeights)+" sum squared weights\n" +
        QString::number(model->massRegular)+" in regular, "+ QString::number(model->massChaotic)+" in chaotic orbits\n"
        );
    else  // signal error
    {
        if(result==-42) emit signalSchwExportNbodyFinished();   // normal exit from ExportNbody
        else if(config.sm_useBPMPD) {
            if(result==-1) emit signalSchwOptimizationFinished("BPMPD: Result data file not found\n"); else
            if(result==-2) emit signalSchwOptimizationFinished("BPMPD: Process failed to start\n"); else
            if(result==-3) emit signalSchwOptimizationFinished("BPMPD: Process crashed unexpectedly\n"); else
            if(result==-13) emit signalSchwOptimizationFinished("BPMPD: Problem size exceeds limit of evaluation version\n"); else
            if(result==-14) emit signalSchwOptimizationFinished("BPMPD: Unrecognized output format\n"); else
                emit signalSchwOptimizationFinished("BPMPD: Unknown error "+QString::number(-2-result)+"\n");
        }
        else
            emit signalSchwOptimizationFinished("GLPK: error "+QString::number(result)+"\n");
    }
    SchwThr->deleteLater();
    SchwThr=NULL;
    if(result==-42 && ConsoleMode) emit scriptNextLine();
}
//// ----------- load/save settings ------------- ////

bool CSmileCore::loadSettings(QString fileName)
{
    if(fileName=="") 
        fileName=AppDir+DEFAULT_SETTINGS_FILE_NAME;
    else if(!QFile::exists(fileName))
        return false;
    QSettings settings(fileName, QSettings::IniFormat);
    QString WorkDir1 = settings.value("Common/WorkDir", "").toString();
    if(WorkDir1!="")
    {
        QFileInfo fi(WorkDir1);
        if(fi.isRelative()) WorkDir = AppDir+WorkDir1; else WorkDir = WorkDir1;
    }
    QString TempDir1 = settings.value("Common/TempDir", "").toString();
    if(TempDir1!="")
    {
        QFileInfo fi(TempDir1);
        if(fi.isRelative()) TempDir = AppDir+TempDir1; else TempDir = TempDir1;
    } 
    else if(TempDir=="") TempDir=WorkDir;
    if(!WorkDir.endsWith('/')) WorkDir += '/';
    if(!TempDir.endsWith('/')) TempDir += '/';

    // potential section
    QString PT = settings.value("Potential/Type", "").toString();
    N_dim = settings.value("Potential/N_dim", N_DIM).toInt();
    if(N_dim<2) N_dim=2;  if(N_dim>3) N_dim=3;
    q = settings.value("Potential/q", q).toDouble(); 
    if(q<0) q=0;  if(q>1) q=1;
    p = settings.value("Potential/p", p).toDouble();
    if(p<0) p=0;  if(p>q) p=q;
    Mbh = settings.value("Potential/Mbh", Mbh).toDouble();
    if(Mbh<0) Mbh=0; 
    Rc = settings.value("Potential/Rc", Rc).toDouble();
    if(Rc<0) Rc=0; 
    Gamma = settings.value("Potential/Gamma", Gamma).toDouble();
    if(Gamma<0) Gamma=0;  if(Gamma>2) Gamma=2;
    Alpha = settings.value("Potential/Alpha", Alpha).toDouble();
    if(Alpha<0.5) Alpha=0.5;
    Ncoefs_radial = settings.value("Potential/Ncoefs_radial", Ncoefs_radial).toDouble();
    if(Ncoefs_radial<=1) Ncoefs_radial=1;
    Ncoefs_angular = settings.value("Potential/Ncoefs_angular", Ncoefs_angular).toDouble();
    if(Ncoefs_angular<=1) Ncoefs_angular=1;
    NbodyFile  = settings.value("Potential/NbodyFile").toString();
    config.nbpotEps   = settings.value("Potential/treecodeEps", TREECODE_DEFAULT_EPS).toDouble();
    config.nbpotTol   = settings.value("Potential/treecodeTheta", TREECODE_DEFAULT_THETA).toDouble();
    config.accuracyRel= settings.value("Potential/accuracyRelative", ACCURACY_REL).toDouble();
    config.accuracyAbs= settings.value("Potential/accuracyAbsolute", ACCURACY_ABS).toDouble();
    config.accTreecode= settings.value("Potential/accuracyTreecode", TREECODE_TIMESTEP_FACTOR).toDouble();
    initPotential(PT);
    // orbit integration section
    icX = settings.value("Orbit/x", 1).toDouble();
    icY = settings.value("Orbit/y", 0).toDouble();
    icZ = settings.value("Orbit/z", 0).toDouble();
    icVx = settings.value("Orbit/vx", 0).toDouble();
    icVy = settings.value("Orbit/vy", 0).toDouble();
    icVz = settings.value("Orbit/vz", 0).toDouble();
    E = settings.value("Orbit/E", 0).toDouble();
    useICe = settings.value("Orbit/useE", false).toBool();
    config.intTimeInPeriods = settings.value("Orbit/intTime", 100).toDouble();
    config.intTimeStepsPerPeriod = settings.value("Orbit/intTimeStep", 50).toDouble();
    config.intTimeMaxAdaptive = settings.value("Orbit/intTimeMax", 0).toDouble();
    config.adaptiveTimeThreshold = settings.value("Orbit/adaptiveTimeThreshold", INTTIME_ADAPTIVE_THRESHOLD).toDouble();
    config.calcLyapunov = settings.value("Orbit/calcLyapunov", false).toBool();
    config.usePS = settings.value("Orbit/usePS", false).toBool();
    // frequency map
    config.fm_numOrbitsStationary = settings.value("Frequency_map/numOrbitsStationary", 768).toInt();
    config.fm_numOrbitsPrincipalPlane = settings.value("Frequency_map/numOrbitsPrincipalPlane", 768).toInt();
    config.fm_numOrbitsYalpha = settings.value("Frequency_map/numOrbitsYalpha", 0).toInt();
    config.fm_numOrbitsRandom = settings.value("Frequency_map/numOrbitsRandom", 0).toInt();
    // schwarzschild modelling
    config.sm_numOrbitsStationary = settings.value("Schwarzschild_model/numOrbitsStationary", 192).toInt();
    config.sm_numOrbitsPrincipalPlane = settings.value("Schwarzschild_model/numOrbitsPrincipalPlane", 192).toInt();
    config.sm_numOrbitsYalpha = settings.value("Schwarzschild_model/numOrbitsYalpha", 0).toInt();
    config.sm_numOrbitsRandom = settings.value("Schwarzschild_model/numOrbitsRandom", 0).toInt();
    config.sm_linesPerSegment = settings.value("Schwarzschild_model/linesPerSegment", 4).toInt();
    config.sm_numShells = settings.value("Schwarzschild_model/numShells", 20).toInt();
    config.sm_numBunchOrbits = settings.value("Schwarzschild_model/numBunchOrbits", 1).toInt();
    config.sm_numSamplingPoints = settings.value("Schwarzschild_model/numSamplingPoints", NUM_SAMPLING_POINTS).toInt();
    config.chaoticMinFreqDiff = settings.value("Schwarzschild_model/chaoticMinFreqDiff", 1e-3).toDouble();
    config.chaoticMinLambda = settings.value("Schwarzschild_model/chaoticMinLambda", 0).toDouble();
    config.chaoticSuperOrbit = settings.value("Schwarzschild_model/chaoticSuperOrbit", 0).toBool();
    config.chaoticWeightFactor = settings.value("Schwarzschild_model/chaoticWeightFactor", 0).toDouble();
    config.maxShellSuperOrbit = settings.value("Schwarzschild_model/maxShellSuperOrbit", 0).toInt();
    config.sm_maxWeight = settings.value("Schwarzschild_model/maxWeight", 0).toDouble();
    config.sm_exportNbodyOrbitsFile = settings.value("Schwarzschild_model/exportNbodyOrbitsFile", 0).toBool();
    config.sm_constrainBeta = settings.value("Schwarzschild_model/constrainBeta", 0).toBool();
    config.sm_betaIn = settings.value("Schwarzschild_model/betaIn", 0).toDouble();
    config.sm_betaOut= settings.value("Schwarzschild_model/betaOut",config.sm_betaIn+0.5).toDouble();
    config.sm_randSigmaXIn = settings.value("Schwarzschild_model/randSigmaXIn", 1.2).toDouble();
    config.sm_randSigmaXOut= settings.value("Schwarzschild_model/randSigmaXOut",1.0).toDouble();
    config.sm_randSigmaYIn = settings.value("Schwarzschild_model/randSigmaYIn", 1.1).toDouble();
    config.sm_randSigmaYOut= settings.value("Schwarzschild_model/randSigmaYOut",0.9).toDouble();
    config.sm_randSigmaZIn = settings.value("Schwarzschild_model/randSigmaZIn", 0.9).toDouble();
    config.sm_randSigmaZOut= settings.value("Schwarzschild_model/randSigmaZOut",0.6).toDouble();
    config.sm_randRefineFrac= settings.value("Schwarzschild_model/randRefineFrac", 0.75).toDouble();
    config.sm_randRefineFact= settings.value("Schwarzschild_model/randRefineFact", 2.0).toDouble();
    config.sm_useBPMPD = settings.value("Schwarzschild_model/useBPMPD", true).toBool();
    if(!QFile::exists(AppDir+"bpmpd.exe")) config.sm_useBPMPD=false;  // by default, use fast BPMPD solver, but if it does not exist, resort to GLPK

    return true;
}

void CSmileCore::saveSettings(QString fileName)
{
    if(fileName=="") fileName=AppDir+DEFAULT_SETTINGS_FILE_NAME;
    QSettings settings(fileName, QSettings::IniFormat);
    // potential section
    settings.setValue("Potential/Type", potential->PotentialName());
    settings.setValue("Potential/N_dim", N_dim);
    settings.setValue("Potential/q", q);
    settings.setValue("Potential/p", p);
    settings.setValue("Potential/Mbh", Mbh);
    settings.setValue("Potential/Rc", Rc);
    settings.setValue("Potential/Gamma", Gamma);
    settings.setValue("Potential/Alpha", Alpha);
    settings.setValue("Potential/Ncoefs_radial", Ncoefs_radial);
    settings.setValue("Potential/Ncoefs_angular", Ncoefs_angular);
    settings.setValue("Potential/NbodyFile", NbodyFile);
    settings.setValue("Potential/treecodeEps", config.nbpotEps);
    settings.setValue("Potential/treecodeTheta", config.nbpotTol);
    settings.setValue("Potential/accuracyRelative", config.accuracyRel);
    settings.setValue("Potential/accuracyAbsolute", config.accuracyAbs);
    settings.setValue("Potential/accuracyTreecode", config.accTreecode);
    // orbit integration section
    settings.setValue("Orbit/x", icX);
    settings.setValue("Orbit/y", icY);
    settings.setValue("Orbit/z", icZ);
    settings.setValue("Orbit/vx", icVx);
    settings.setValue("Orbit/vy", icVy);
    settings.setValue("Orbit/vz", icVz);
    settings.setValue("Orbit/E", E);
    settings.setValue("Orbit/useE", useICe);
    settings.setValue("Orbit/intTime", config.intTimeInPeriods);
    settings.setValue("Orbit/intTimeStep", config.intTimeStepsPerPeriod);
    settings.setValue("Orbit/intTimeMax", config.intTimeMaxAdaptive);
    settings.setValue("Orbit/adaptiveTimeThreshold", config.adaptiveTimeThreshold);
    settings.setValue("Orbit/calcLyapunov", config.calcLyapunov);
    settings.setValue("Orbit/usePS", config.usePS);
    // frequency map
    settings.setValue("Frequency_map/numOrbitsStationary", config.fm_numOrbitsStationary);
    settings.setValue("Frequency_map/numOrbitsPrincipalPlane", config.fm_numOrbitsPrincipalPlane);
    settings.setValue("Frequency_map/numOrbitsYalpha", config.fm_numOrbitsYalpha);
    settings.setValue("Frequency_map/numOrbitsRandom", config.fm_numOrbitsRandom);
    // schwarzschild modelling
    settings.setValue("Schwarzschild_model/numOrbitsStationary", config.sm_numOrbitsStationary);
    settings.setValue("Schwarzschild_model/numOrbitsPrincipalPlane", config.sm_numOrbitsPrincipalPlane);
    settings.setValue("Schwarzschild_model/numOrbitsYalpha", config.sm_numOrbitsYalpha);
    settings.setValue("Schwarzschild_model/numOrbitsRandom", config.sm_numOrbitsRandom);
    settings.setValue("Schwarzschild_model/linesPerSegment", config.sm_linesPerSegment);
    settings.setValue("Schwarzschild_model/numShells", config.sm_numShells);
    settings.setValue("Schwarzschild_model/numBunchOrbits", config.sm_numBunchOrbits);
    settings.setValue("Schwarzschild_model/numSamplingPoints", config.sm_numSamplingPoints);
    settings.setValue("Schwarzschild_model/chaoticMinFreqDiff", config.chaoticMinFreqDiff);
    settings.setValue("Schwarzschild_model/chaoticMinLambda", config.chaoticMinLambda);
    settings.setValue("Schwarzschild_model/chaoticSuperOrbit", config.chaoticSuperOrbit);
    settings.setValue("Schwarzschild_model/chaoticWeightFactor", config.chaoticWeightFactor);
    settings.setValue("Schwarzschild_model/maxShellSuperOrbit", config.maxShellSuperOrbit);
    settings.setValue("Schwarzschild_model/constrainBeta", config.sm_constrainBeta);
    settings.setValue("Schwarzschild_model/betaIn",  config.sm_betaIn);
    settings.setValue("Schwarzschild_model/betaOut", config.sm_betaOut);
    settings.setValue("Schwarzschild_model/randSigmaXIn",  config.sm_randSigmaXIn);
    settings.setValue("Schwarzschild_model/randSigmaXOut", config.sm_randSigmaXOut);
    settings.setValue("Schwarzschild_model/randSigmaYIn",  config.sm_randSigmaYIn);
    settings.setValue("Schwarzschild_model/randSigmaYOut", config.sm_randSigmaYOut);
    settings.setValue("Schwarzschild_model/randSigmaZIn",  config.sm_randSigmaZIn);
    settings.setValue("Schwarzschild_model/randSigmaZOut", config.sm_randSigmaZOut);
    settings.setValue("Schwarzschild_model/randRefineFrac", config.sm_randRefineFrac);
    settings.setValue("Schwarzschild_model/randRefineFact", config.sm_randRefineFact);
    settings.setValue("Schwarzschild_model/exportNbodyOrbitsFile", config.sm_exportNbodyOrbitsFile);
    settings.setValue("Schwarzschild_model/maxWeight", config.sm_maxWeight);
}

QString CSmileCore::getDirName(const QString &fileName)  // return file path ending with "/"
{
    QFileInfo fi(fileName);
    QString fn = fi.absolutePath();
    if(!fn.endsWith('/')) fn += '/';
    return fn;
}

//// ----------- console scripting ------------ //// 

void CSmileCore::runScript(QString scriptFileName)
{
    ConsoleMode=true;
    std::cout << "SMILE console version  "VERSION", "__DATE__ << std::endl;
    initIC();
	if(scriptFileName!="")
	{
		input.setFileName(scriptFileName);
        ConsoleInput=false;
        if(!input.open(QIODevice::ReadOnly | QIODevice::Text))
	    {
		    std::cout << "Error! cannot read script file " << scriptFileName.toStdString() << std::endl;
		    emit scriptDone();
	    }
	}
	else
    {
        input.open(stdin, QIODevice::ReadOnly | QIODevice::Text);
        ConsoleInput=true;
    }
    connect(this, SIGNAL(scriptNextLine()), SLOT(scriptProcessLine()));
    connect(this, SIGNAL(signalFreqMapFinished()), SLOT(scriptOrbitLibraryFinished()));
    connect(this, SIGNAL(signalSchwOrbitLibraryFinished()), SLOT(scriptOrbitLibraryFinished()));
    connect(this, SIGNAL(signalSchwOptimizationFinished(const QString&)), SLOT(scriptOptimizationFinished(const QString&)));
    connect(this, SIGNAL(signalInfo(const QString&)), SLOT(scriptInfo(const QString&)));
    connect(this, SIGNAL(signalTimer()), SLOT(scriptTimerEvent()));
    timeRuntime.start();
    myTimer=0;
    emit scriptNextLine();
}

void CSmileCore::scriptInfo(const QString& message)
{
    std::cout << message.toStdString() << std::endl;
}

void CSmileCore::timerEvent(QTimerEvent *)
{
    emit signalTimer();
}

void CSmileCore::scriptTimerEvent()
{
    if(!orbits) return;
    std::cout << orbits->nComplete << " out of " << orbits->OrbitList.size() << " orbits done\015" << std::flush;
}

void CSmileCore::scriptOrbitLibraryFinished()
{
    std::cout << orbits->nComplete << " orbits total, " << orbits->numChaotic << " chaotic, " << 
        orbits->nComplete*1000.0/timeElapsed.elapsed() << " orbits/s" << std::endl;
    if(myTimer!=0){
        killTimer(myTimer);
        myTimer=0;
    }
    emit scriptNextLine();
}

void CSmileCore::scriptOptimizationFinished(const QString& message)
{
    emit scriptInfo(message);
    emit scriptNextLine();
}

void CSmileCore::scriptProcessLine()
{
    QString Line;
    std::cout << "> ";
	if(!input.atEnd())
		Line = input.readLine().trimmed();
    else
        Line = "Exit";
    if(Line=="" || Line.startsWith("#"))
    {
        emit scriptNextLine();
        return;
    }
    if(Line.startsWith("Exit", Qt::CaseInsensitive) || Line.startsWith("Quit", Qt::CaseInsensitive) || Line.startsWith("Bye", Qt::CaseInsensitive))
    {
        std::cout << "Done!  Total runtime: " << timeRuntime.elapsed()/1000 << " seconds" << std::endl;
        emit scriptDone();
        return;
    }
    if(!ConsoleInput) 
        std::cout << (Line.toStdString()) << std::endl;
    
    // parse input string and perform desired action
    if(Line.startsWith("ReadIni", Qt::CaseInsensitive) || Line.startsWith("ReadConfig", Qt::CaseInsensitive) || Line.startsWith("LoadIni", Qt::CaseInsensitive) || Line.startsWith("LoadConfig", Qt::CaseInsensitive))
    {
        int i1=Line.indexOf("(""");
        int i2=Line.indexOf(""")");
        QString FileName = Line.mid(i1+2, i2-i1-3);
        if(i1>0 && i2>i1)
        {
            if(loadSettings(FileName))
            {
                std::cout << "Loaded config from " << FileName.toStdString() << std::endl;
               // delete orbits;
               // orbits=NULL;
                initIC();
            }
            else
                std::cout << "Error! Cannot load config from " << FileName.toStdString() << std::endl;
        }
        else std::cout << "Error! No filename specified" << std::endl;
    }
    else if(Line.startsWith("ExportPotential", Qt::CaseInsensitive))
    {
        int i1=Line.indexOf("(""");
        int i2=Line.indexOf(""")");
        QString FileName = Line.mid(i1+2, i2-i1-3);
        if(i1>0 && i2>i1)
        {
            if(exportPotential(FileName))
                std::cout << "Potential exported to " << FileName.toStdString() << std::endl;
            else
                std::cout << "Error! Cannot export potential to " << FileName.toStdString() << std::endl;
        }
        else std::cout << "Error! No filename specified" << std::endl;
    }
    else if(Line.startsWith("BuildFreqMap", Qt::CaseInsensitive))
    {
        BuildFreqMapIC(E, Line.contains("exist", Qt::CaseInsensitive));
        StartOrbitLibrary(SLOT(coreFreqMapFinished()));
        timeElapsed.start();
        return;
    }
    else if(Line.startsWith("SchwBuildOrbitLibrary", Qt::CaseInsensitive))
    {
        BuildSchwIC(Line.contains("exist", Qt::CaseInsensitive));
        StartOrbitLibrary(SLOT(coreSchwOrbitLibraryFinished()));
        timeElapsed.start();
        return;
    }
    else if(Line.startsWith("SchwLinearOptimization", Qt::CaseInsensitive) || Line.startsWith("SchwQuadraticOptimization", Qt::CaseInsensitive) || Line.startsWith("SchwLucyOptimization", Qt::CaseInsensitive))
    {
        if(model!=NULL && orbits!=NULL)
        {
            CSchwarzschildThread::OPERATION operation = CSchwarzschildThread::SM_LINEAR;
            if(Line.startsWith("SchwQuadraticOptimization")) operation=CSchwarzschildThread::SM_QUADRATIC;
            if(Line.startsWith("SchwLucyOptimization")) operation=CSchwarzschildThread::SM_LUCY;
            SchwStartOptimization(operation);
            return;
        }
        else
            std::cout << "No Schwarzschild model or orbit library defined" << std::endl;
    }
    else if(Line.startsWith("ExportOrbits", Qt::CaseInsensitive))
    {
        if(orbits)
        {
            int i1=Line.indexOf("(""");
            int i2=Line.indexOf(""")");
            QString FileName = Line.mid(i1+2, i2-i1-3);
            if(i1>0 && i2>i1)
            {
                if(!orbits->exportOrbits(FileName, Line.startsWith("ExportOrbitsCell", Qt::CaseInsensitive)))
                    std::cout << "Error writing file: " << FileName.toStdString() << std::endl;
            }
            else std::cout << "No filename specified" << std::endl;
        }
        else
            std::cout << "No orbit library defined" << std::endl;
    }
    else if(Line.startsWith("ImportOrbits", Qt::CaseInsensitive))
    {
        int i1=Line.indexOf("(""");
        int i2=Line.indexOf(""")");
        QString FileName = Line.mid(i1+2, i2-i1-3);
        if(i1>0 && i2>i1)
        {
            if(orbits)
                delete orbits;
            bool sm=Line.startsWith("ImportOrbitsCell", Qt::CaseInsensitive);
            if(sm)
            {
                if(model) delete model;
                model=new CSchwarzschildModel(potential, config.sm_numShells, config.sm_linesPerSegment);
            }
            orbits=new COrbitLibrary(potential, sm?model->grid:NULL);
            if(!orbits->importOrbits(FileName, sm))
                std::cout << "Error reading file: " << FileName.toStdString() << std::endl;
            else
            {
                std::cout << orbits->nComplete << " orbits loaded" << std::endl;
                if(sm) model->orbits=orbits;
            }
        }
        else std::cout << "No filename specified" << std::endl;
    }
    else if(Line.startsWith("SchwExportGrid", Qt::CaseInsensitive) || Line.startsWith("ExportGrid", Qt::CaseInsensitive))
    {
        if(model)
        {
            int i1=Line.indexOf("(""");
            int i2=Line.indexOf(""")");
            QString FileName = Line.mid(i1+2, i2-i1-3);
            if(i1>0 && i2>i1)
            {
                if(!model->exportGrid(FileName))
                    std::cout << "Error writing file: " << FileName.toStdString() << std::endl;
            }
            else std::cout << "No filename specified" << std::endl;
        }
        else
            std::cout << "No Schwarzschild model defined" << std::endl;
    }
    else if(Line.startsWith("ExportNbody", Qt::CaseInsensitive))
    {
        if(orbits && model)
        {
            int i1=Line.indexOf("(");
            int i2=Line.lastIndexOf(")");
            QString paramstr = Line.mid(i1+1, i2-i1-1);
            QStringList params=paramstr.split(QRegExp("[,\\s]+"));
            if(params.count()<2)
                std::cout << "Syntax: ExportNbody(NumBodies, \"FileName\"[, RefineFactor=0])" << std::endl;
            else{
                int NPoints=params[0].toInt();
                i1=params[1].indexOf("\"");
                i2=params[1].lastIndexOf("\"");
                QString fileName=params[1].mid(i1+1, i2-i1-1);
                int refineFactor=0;
                if(params.count()>2)
                    refineFactor=params[2].toInt();
                if(NPoints==0)
                    std::cout << "Incorrect first parameter: NumBodies" << std::endl;
                else
                if(fileName!="")
                {
                    SchwExportNbody(NPoints, fileName, refineFactor);
                    return;
                }
                else std::cout << "No filename specified" << std::endl;
            }
        }
        else
            std::cout << "No orbit library or Schwarzschild model defined" << std::endl;
    }
    else 
        std::cout << "Invalid command" << std::endl;
    emit scriptNextLine();
}

///////////////////////////////////// Helper threads that run various tasks in parallel to main program /////

// CalcThread - runs single orbit integration
void CCalcThread::run()
{
    orbit->integrateToTime(time);
    orbit->finishIntegration();
    emit calcFinished();
}

// CalcManyThr - runs orbit library integration in parallel
volatile int nthreads=0;
QMutex mutexL;
QMutex mutexC;

void CCalcManyThread::run()
{
    mutexC.lock();
    ++nthreads;
    mutexC.unlock();
    finished=false;
    while(!finished)
    {
        int no=-1;
        mutexL.lock();
        int nComplete=0;
        for(int i=0; (i<orbits->OrbitList.size())&&(no<0); i++)
            if(orbits->OrbitList[i].state==0) 
            {
                no=i;
                orbits->OrbitList[no].state=1;
            }
            else if(orbits->OrbitList[i].state==2)
                ++nComplete;
        orbits->nComplete=nComplete;
        mutexL.unlock();
        if(no>=0)
        {
            orbits->OrbitList[no].run();
            mutexL.lock();
            ++orbits->nComplete;
            mutexL.unlock();
        }
        else finished=true;
    }
    mutexC.lock();
    --nthreads;
    if(nthreads==0)
        emit calcFinished();
    mutexC.unlock();
}

void CCalcManyThread::stopThread()
{
    finished=true;
}

// SchwarzschildThread - runs various tasks related to Schw.modelling (optimization and Nbody export)
void CSchwarzschildThread::run()
{
    connect(model, SIGNAL(calcProgress(const QString&)), SIGNAL(calcProgress(const QString&)));
    double result=0;
    switch(operation)
    {
    case SM_LINEAR: 
    case SM_QUADRATIC:
        nIter=-1;
        result = model->solveOptimizationProblem(operation==SM_QUADRATIC);
        break;
    case SM_LUCY:
        result = model->solveLucyIterationsOptimization(nIter);
        break;
    case SM_EXPORTNBODY:
        model->exportNbody();
        result=-42;
        break;
    }
    emit calcFinished(result);
}

}