#pragma once
#include <QObject>
#include <QStringList>
#include <QTime>
#include <QFile>
#include <QTextStream>
#include <vector>

#include "orbit.h"
#include "schwarzschild.h"

namespace smile{

///////////////////////////////////// Helper threads that run various tasks in parallel to main program /////

// Thread that runs orbit integration 
class CCalcThread : public QThread
{
    Q_OBJECT
public:
    void run();
    CCalcThread(COrbit* _orbit, double _time): orbit(_orbit), time(_time) {};
signals:
    void calcFinished();
private:
    COrbit *orbit;   // do not copy orbit - store only pointer
    double time;
};

// Thread that runs orbit library integration (several threads in parallel)
class CCalcManyThread : public QThread
{
    Q_OBJECT
public:
    void run();
    CCalcManyThread(COrbitLibrary* _orbits): orbits(_orbits) {};
signals:
    void calcFinished();
public slots:
    void stopThread();
private:
    COrbitLibrary* orbits;
    bool finished;
};

// Thread that runs Schwarzschild modelling 
class CSchwarzschildThread : public QThread
{
    Q_OBJECT
public:
enum OPERATION
{
    SM_LINEAR,
    SM_QUADRATIC,
    SM_LUCY,
    SM_EXPORTNBODY
};
    CSchwarzschildThread(CSchwarzschildModel* _model, OPERATION _operation): 
        model(_model), operation(_operation) {};
    void run();
    int nIter;  // keeps number of Lucy iterations
signals:
    void calcProgress(const QString& message);
    void calcFinished(double result);
private:
    OPERATION operation;
    CSchwarzschildModel* model;
};

//////////////////////////////////// CSmileCore class holds all non-gui workflow ////

class CSmileCore: public QObject
{
    Q_OBJECT

public:
    // --- configuration data --- //
    // potential data
    int N_dim;
    double q, p, Mbh, Rc, Gamma, Alpha;  // these values will be used to initialize proper potential type
    int Ncoefs_radial, Ncoefs_angular;   // number of radial and angular coefficients in spherical-harmonic expansion
    QString NbodyFile;               // name of file containing Nbody coordinates to calculate spherical-harmonic coefficients
    CPotential::POTENTIALTYPE PotentialType; // currently selected potential type
    // Single orbit data
    double icX, icY, icZ, icVx, icVy, icVz;  // initial position and velocity
    double E;                        // energy 
    bool useICe;                     // use energy as an initial condition instead of position/velocity (applies to freq.map)
    QString WorkDir, TempDir, AppDir;// directories: current working directory (for GUI file dialogs); tempdir - for exchanging data with external programs (qdelaunay, bpmpd); appdir - where to locate these programs
    int maxNumThread;                // equal to number of processors/cores on the machine
    // --- end of configuration data --- //

    CPotential* potential;           // potential currently used in all calculations

    COrbit* orbit;                   // current orbit
    double Xmax, Torbx;              // radius and period of x-axis orbit for given initial conditions
    double maxdist;                  // max. distance between adjacent points of trajectory (used in triangulation thread)
    double maxr;                     // max. orbit radius (used in GUI plots to set axis scale

    COrbitLibrary* orbits;           // library of orbits (for frequency map and Schwarzschild model)
    CSchwarzschildModel* model;      // model

    CCalcThread* CalcThr;            // thread that performs orbit integration asynchronously
    std::vector<CCalcManyThread*> CalcManyThr;  // pool of threads for calculating orbits
    CSchwarzschildThread* SchwThr;   // thread that does Schwarzschild modelling
    int myTimer;                     // timer object for displaying progress in orbit library integration
    QTime timeElapsed, timeRuntime;  // measure time for orbit integration and for total runtime
    QFile input;                     // input file for scripting
    bool ConsoleMode, ConsoleInput;  // set when running console version; if interactive, set ConsoleInput=true, if processing script, set to false

    CSmileCore();                              // create the instance and set global variable 'core'
    bool loadSettings(QString fileName="");    // load configuration data (at start - from predefined file smile.ini, later - when loading orbit library
    void saveSettings(QString fileName="");    // store configuration data (when storing orbit library, or at exit, if demanded in GUI)
    void runScript(QString scriptFileName=""); // in console mode: run script file or process data from console
    // potential
    void initPotential(QString potentialName); // create instance of a potential of given type
    void initPotential(CPotential::POTENTIALTYPE potentialType);
    void initPotentialNB(bool frozenN);    // load Nbody coords file, if frozenN=true, use direct N-body potential, else calculate density and potential in basis set expansion
    // orbit integration
    bool initIC();                         // set up initial conditions (calc E from X,Y,Z or vice-versa, calc Torbx, etc.) Return true if something 
    void startOrbit();                     // start thread that runs orbit integration
    bool exportOrbit(QString fileName);    // export orbit into a text file
    bool exportPotential(QString fileName);      // export potential file (values of potential and forces in certain points of space, for statistics)
    // Frequency map and Schwarzschild modelling
    void BuildFreqMapIC(double energy, bool useExistingIC);  // create initial conditions for orbit library for frequency map at given energy
    void BuildSchwIC(bool useExistingIC);                    // create initial conditions for orbit library for Schwarzschild model
    void StartOrbitLibrary(const char* finishSignal);        // start several threads to integrate orbits from orbit library
    void SchwStartOptimization(CSchwarzschildThread::OPERATION operation);      // start thread that solves Schw.optimization problem
    void SchwExportNbody(int numPoints, QString fileName, int refineFactor=0);  // export Schw model to Nbody model
    virtual void timerEvent(QTimerEvent*); // to display progress indicator in orbit library integration
    QString getDirName(const QString &fileName);  // convenience function, return file path ending with "/"

signals:
    void signalInfo(const QString&);       // display various information (GUI - in message area, console - print out string)
    void signalTimer();                    // emitted regularly during orbit library integration, to update progress info
    void signalOrbitFinished();            // emitted when orbit integration finished
    void signalFreqMapFinished();          // emitted upon finish of frequency map building
    void signalSchwOrbitLibraryFinished(); // emitted upon finish of building orbit library for Schwarzschild modelling
    void signalSchwOptimizationFinished(const QString&);  // emitted when Schwarzschild optimization finished
    void signalSchwExportNbodyFinished();  // emitted when export to nbody model is finished
    void scriptNextLine();                 // emitted upon having processed a line in console script
    void scriptDone();                     // emitted when finished processing console script

private slots:
    void coreOrbitFinished();              // called when CalcThread finishes orbit integration
    void coreFreqMapFinished();            // called upon finish of last thread in frequency map calculation; clears unused orbits and emits signalFreqMapFinished
    void coreSchwOrbitLibraryFinished();   // called upon finish of last thread in building orbit library for Schwarzschild modelling; clears unused orbits and emits signalSchwOrbitLibraryFinished
    void coreSchwOptimizationFinished(double result);  // called upon finish of Schwarzschild optimization thread; destroys thread and emits signalSchwOptimizationFinished
    void coreSchwExportNbody();            // called when reintegration of orbit library finished, exports nbody IC
    void scriptProcessLine();              // processes next line in a script 
    void scriptOrbitLibraryFinished();     // displays some statistics (in console mode is connected to signalFreqMapFinished and signalSchwOrbitLibraryFinished)
    void scriptOptimizationFinished(const QString& message);  // displays results of optimization in console mode
    void scriptTimerEvent();               // displays progress in orbit library integration in console mode
    void scriptInfo(const QString& message);  // displays message in console mode
#ifdef FPI
    void makeFPImajorstep();
#endif
};

extern CSmileCore* core;   // the instance of core object holding all non-gui data

}