#include <QKeyEvent>
#include <QFile>
#include <QTextStream>
#include <QFileDialog>
#include <QMessageBox>
#include <QPrinter>
#include <QImage>
#include <QMap>
#include <QInputDialog>

#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qwt_text.h>
#include <qwt_symbol.h>
#include <qwt_plot_picker.h>
#include <qwt_plot_panner.h>
#include <qwt_plot_zoomer.h>
#include <qwt_scale_engine.h>
#include <qwt_plot_marker.h>

#include <cmath>
#include <vector>
#include <algorithm>

#include "gui.h"
#include "core.h"

#ifdef USE3DFACETS
// for triangulation thread
#include <QVector>
#include <QProcess>
#include <QTextStream>
#endif

namespace smile{

CSmileGUI::CSmileGUI(QWidget *parent, Qt::WFlags flags)
    : QMainWindow(parent, flags)
{
	ui.setupUi(this);

    new CSmileCore();  // initialize instance of global object holding the core of calculation procedures
    // connect signals to the core
    connect(core, SIGNAL(signalOrbitFinished()), SLOT(CalcFinished()));
    connect(core, SIGNAL(signalFreqMapFinished()), SLOT(FreqMapFinished()));
    connect(core, SIGNAL(signalSchwOrbitLibraryFinished()), SLOT(SchwOrbitLibraryFinished()));
    connect(core, SIGNAL(signalSchwOptimizationFinished(const QString&)), SLOT(SchwOptimizationFinished(const QString&)));
    connect(core, SIGNAL(signalSchwExportNbodyFinished()), SLOT(SchwExportNbodyFinished()));
    connect(core, SIGNAL(signalInfo(const QString&)), SLOT(info(const QString&)));
    connect(core, SIGNAL(signalTimer()), SLOT(timerEventCore()));
    
    Eprev=0;
    FreqMapRunning=false;
    SchwModelRunning=false;
    ExportNbodyRunning=false;
    
    // orbit plot
    PlotOrbit = new QwtPlot(QwtText("Orbit"), ui.frame_orbitarea);
    CurveOrbit= new QwtPlotCurve("");
    CurveOrbit->setPen(QColor(Qt::blue));
    CurveOrbit->attach(PlotOrbit);
    CurveEquipotential= new QwtPlotCurve("");
    CurveEquipotential->setPen(QColor(Qt::red));
    CurveEquipotential->attach(PlotOrbit);
    PlotOrbit->show();
    QVBoxLayout* LPlotOrbit = new QVBoxLayout(ui.frame_orbitarea);
    LPlotOrbit->addWidget(PlotOrbit);
    PlotOrbitParam = new QwtPlot(QwtText("Distribution of orbital parameters"), ui.frame_orbitarea);
    CurveOrbitParam= new QwtPlotCurve("");
    CurveOrbitParam->attach(PlotOrbitParam);
    PlotOrbitParam->hide();
    LPlotOrbit->addWidget(PlotOrbitParam);
    ZoomerOrbitParam= new QwtPlotZoomer( QwtPlot::xBottom, QwtPlot::yLeft, 
        QwtPicker::DragSelection | QwtPicker::CornerToCorner, QwtPicker::ActiveOnly, PlotOrbitParam->canvas(), true);
    ZoomerOrbitParam->setMousePattern(QwtEventPattern::MouseSelect3, Qt::RightButton);
    QwtPlotPanner* PannerOrbitParam = new QwtPlotPanner(PlotOrbitParam->canvas());
    PannerOrbitParam->setMouseButton(Qt::MidButton);

#ifdef USEQWT3D
    // 3d orbit
    Plot3d=new SurfacePlot(ui.frame_orbitarea);
    Plot3d->setCoordinateStyle(Qwt3D::FRAME);
    Plot3d->setRotation(90,0,0);
    QColor bkg=ui.tabWidget->palette().color(QPalette::Background);
    Plot3d->setBackgroundColor(Qwt3D::RGBA(bkg.redF(), bkg.greenF(), bkg.blueF()));
    Plot3d->setMeshColor(Qwt3D::RGBA(0.2,0,0.4));
    for (unsigned i=0; i!=Plot3d->coordinates()->axes.size(); ++i)
    {
      Plot3d->coordinates()->axes[i].setMajors(7);
      Plot3d->coordinates()->axes[i].setMinors(4);
    }
    LPlotOrbit->addWidget(Plot3d);
    ui.progressBarOrbit->hide();
#ifdef USE3DFACETS
    TriangThr = new CTriangThread();
    connect(TriangThr, SIGNAL(finished(bool)), SLOT(redrawOrbit(bool)));
    Plot3d->setLightComponent(GL_DIFFUSE, 0.3);
    Plot3d->setMaterialComponent(GL_SPECULAR, 0.0);
    Plot3d->enableLighting();
    Plot3d->illuminate();
    Plot3d->updateGL();
#endif
#endif

    // poincare plot
    PlotPoincare = new QwtPlot(QwtText("Poincare section"), ui.frame_poincarearea);
    PlotPoincare->show();
    QVBoxLayout * LPlotPoincare = new QVBoxLayout(ui.frame_poincarearea);
    LPlotPoincare->addWidget(PlotPoincare);
    QwtPlotPicker *PickerPoincare = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection, QwtPlotPicker::NoRubberBand, QwtPicker::AlwaysOn, PlotPoincare->canvas());
    PickerPoincare->setMousePattern(QwtEventPattern::MouseSelect1, Qt::RightButton);
    connect(PickerPoincare, SIGNAL(selected(const QwtDoublePoint &)), SLOT(PSclicked(const QwtDoublePoint &)));
    ZoomerPoincare = new QwtPlotZoomer( QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::DragSelection | QwtPicker::CornerToCorner, QwtPicker::AlwaysOn, PlotPoincare->canvas(), true);
    ZoomerPoincare->setMousePattern(QwtEventPattern::MouseSelect3, Qt::RightButton, Qt::ControlModifier);
    ZoomerPoincare->setMousePattern(QwtEventPattern::MouseSelect2, Qt::RightButton, Qt::ControlModifier);
    ZoomerPoincare->setKeyPattern(QwtEventPattern::KeySelect1, Qt::Key_Space);
    QwtPlotPanner* PannerPoincare = new QwtPlotPanner(PlotPoincare->canvas());
    PannerPoincare->setMouseButton(Qt::MidButton);

    // frequency plot
    PlotFreq = new QwtPlot(QwtText("Frequencies"), ui.frame_frequencies);
    CurveFreqX= new QwtPlotCurve("X");
    CurveFreqX->setPen(QColor(Qt::blue));
    CurveFreqX->attach(PlotFreq);
    CurveFreqY= new QwtPlotCurve("Y");
    CurveFreqY->setPen(QColor(Qt::darkGreen));
    CurveFreqY->attach(PlotFreq);
    CurveFreqZ= new QwtPlotCurve("Z");
    CurveFreqZ->setPen(QColor(Qt::darkRed));
    CurveFreqZ->attach(PlotFreq);
    CurveFreqPeaks= new QwtPlotCurve();
    CurveFreqPeaks->setStyle(QwtPlotCurve::Sticks);
    CurveFreqPeaks->attach(PlotFreq);
    CurveFreqPeaksP= new QwtPlotCurve();
    CurveFreqPeaksP->setStyle(QwtPlotCurve::Sticks);
    CurveFreqPeaksP->setPen(QColor(Qt::white));
    CurveFreqPeaksP->attach(PlotFreq);
    PlotFreq->show();
    QVBoxLayout* LPlotFreq = new QVBoxLayout(ui.frame_frequencies);
    LPlotFreq->addWidget(PlotFreq);
    ZoomerFreq = new QwtPlotZoomer( QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::DragSelection | QwtPicker::CornerToCorner, QwtPicker::AlwaysOn, PlotFreq->canvas(), true);
    ZoomerFreq->setMousePattern(QwtEventPattern::MouseSelect3, Qt::RightButton);
    QwtPlotPanner* PannerFreq = new QwtPlotPanner(PlotFreq->canvas());
    PannerFreq->setMouseButton(Qt::MidButton);

    // Lyapunov plot
    PlotLyapunov = new QwtPlot(QwtText("Lyapunov number"), ui.tab_l);
    PlotLyapunov->setAxisScaleEngine(QwtPlot::xBottom, new QwtLog10ScaleEngine);
    PlotLyapunov->setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine);
    PlotLyapunov->setAxisScaleEngine(QwtPlot::yRight, new QwtLog10ScaleEngine);
    CurveDeviation = new QwtPlotCurve("");
    CurveDeviation->setPen(QColor(Qt::red));
    CurveDeviation->attach(PlotLyapunov);
    CurveDeviation->setYAxis(QwtPlot::yRight);
    PlotLyapunov->enableAxis(QwtPlot::yRight);
    CurveLyapunov = new QwtPlotCurve("");
    CurveLyapunov->setPen(QColor(Qt::blue));
    CurveLyapunov->attach(PlotLyapunov);
    PlotLyapunov->show();
    QwtPlotPicker *PickerLyapunov = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection, QwtPlotPicker::NoRubberBand, QwtPicker::AlwaysOn, PlotLyapunov->canvas());
    QVBoxLayout* LPlotLyapunov = new QVBoxLayout(ui.tab_l);
    LPlotLyapunov->addWidget(PlotLyapunov);

    // Frequency Map plot
    PlotFreqMap = new QwtPlot(QwtText("Frequency map"), ui.frame_fmarea);
    QwtSymbol sym;
    sym.setStyle(QwtSymbol::Rect);
    sym.setSize(2);
    sym.setPen(Qt::NoPen);
    sym.setBrush(QColor(Qt::red));
    CurveFreqMapC = new QwtPlotCurve("");
    CurveFreqMapC->setSymbol(sym);
    CurveFreqMapC->setStyle(QwtPlotCurve::Dots);
    CurveFreqMapC->attach(PlotFreqMap);
    sym.setBrush(QColor(Qt::blue));
    CurveFreqMapR = new QwtPlotCurve("");
    CurveFreqMapR->setSymbol(sym);
    CurveFreqMapR->setStyle(QwtPlotCurve::Dots);
    CurveFreqMapR->attach(PlotFreqMap);
    drawFreqMapResonances();
    PlotFreqMap->show();
    QVBoxLayout* LPlotFreqMap = new QVBoxLayout(ui.frame_fmarea);
    LPlotFreqMap->addWidget(PlotFreqMap);
    QwtPlotPicker *PickerFreqMap = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection, QwtPlotPicker::NoRubberBand, QwtPicker::AlwaysOn, PlotFreqMap->canvas());
    PickerFreqMap->setMousePattern(QwtEventPattern::MouseSelect1, Qt::RightButton);
    connect(PickerFreqMap, SIGNAL(selected(const QwtDoublePoint &)), SLOT(FMclicked(const QwtDoublePoint &)));
    ZoomerFreqMap = new QwtPlotZoomer( QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::DragSelection | QwtPicker::CornerToCorner, QwtPicker::AlwaysOn, PlotFreqMap->canvas(), true);
    ZoomerFreqMap->setMousePattern(QwtEventPattern::MouseSelect3, Qt::RightButton, Qt::ControlModifier);
    ZoomerFreqMap->setMousePattern(QwtEventPattern::MouseSelect2, Qt::RightButton, Qt::ControlModifier);
    ZoomerFreqMap->setKeyPattern(QwtEventPattern::KeySelect1, Qt::Key_Space);
    QwtPlotPanner* PannerFreqMap = new QwtPlotPanner(PlotFreqMap->canvas());
    PannerFreqMap->setMouseButton(Qt::MidButton);

    PlotFreqMapHisto = new QwtPlot(QwtText("Histograms"), ui.frame_fmarea);
    PlotFreqMapHisto->setAxisScaleEngine(QwtPlot::xBottom, new QwtLog10ScaleEngine);
    CurveFreqMapHisto = new QwtPlotCurve("");
    CurveFreqMapHisto->setStyle(QwtPlotCurve::Steps);
    CurveFreqMapHisto->attach(PlotFreqMapHisto);
    LPlotFreqMap->addWidget(PlotFreqMapHisto);
    QwtPlotPicker *PickerFreqMapHisto = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection, QwtPlotPicker::NoRubberBand, QwtPicker::AlwaysOn, PlotFreqMapHisto->canvas());
    PlotFreqMapHisto->hide();

    PlotFreqMapSS = new QwtPlot(QwtText("Start-space"), ui.frame_fmarea);
    sym.setStyle(QwtSymbol::Rect);
    sym.setSize(4);
    sym.setBrush(QColor(Qt::blue));
    CurveFreqMapSSR = new QwtPlotCurve("");
    CurveFreqMapSSR->setSymbol(sym);
    CurveFreqMapSSR->setStyle(QwtPlotCurve::Dots);
    CurveFreqMapSSR->attach(PlotFreqMapSS);
    sym.setBrush(QColor(Qt::red));
    CurveFreqMapSSC = new QwtPlotCurve("");
    CurveFreqMapSSC->setSymbol(sym);
    CurveFreqMapSSC->setStyle(QwtPlotCurve::Dots);
    CurveFreqMapSSC->attach(PlotFreqMapSS);
    QwtPlotPicker *PickerFreqMapSS = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection, QwtPlotPicker::NoRubberBand, QwtPicker::ActiveOnly, PlotFreqMapSS->canvas());
    PickerFreqMapSS->setMousePattern(QwtEventPattern::MouseSelect1, Qt::RightButton);
    connect(PickerFreqMapSS, SIGNAL(selected(const QwtDoublePoint &)), SLOT(FMclicked(const QwtDoublePoint &)));
    LPlotFreqMap->addWidget(PlotFreqMapSS);
    PlotFreqMapSS->hide();
    ui.progressBarFM->hide();

    // Schwarzschild plot
    QVBoxLayout* LPlotSchw = new QVBoxLayout(ui.frame_smarea);
    PlotSchw = new QwtPlot(QwtText("Schwarzschild model grid"), ui.frame_smarea);
    sym.setStyle(QwtSymbol::Diamond);
    sym.setSize(8);
    sym.setBrush(QColor(Qt::blue));
    CurveSchwCellF = new QwtPlotCurve("");
    CurveSchwCellF->setSymbol(sym);
    CurveSchwCellF->setStyle(QwtPlotCurve::Dots);
    CurveSchwCellF->attach(PlotSchw);
    sym.setBrush(QColor(Qt::red));
    CurveSchwCellI= new QwtPlotCurve("");
    CurveSchwCellI->setSymbol(sym);
    CurveSchwCellI->setStyle(QwtPlotCurve::Dots);
    CurveSchwCellI->attach(PlotSchw);
    QwtPlotPicker *PickerFreqMapSM = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection, QwtPlotPicker::NoRubberBand, QwtPicker::ActiveOnly, PlotSchw->canvas());
    PickerFreqMapSM->setMousePattern(QwtEventPattern::MouseSelect1, Qt::RightButton);
    connect(PickerFreqMapSM, SIGNAL(selected(const QwtDoublePoint &)), SLOT(SMclicked(const QwtDoublePoint &)));
    PlotSchw->show();
    LPlotSchw->addWidget(PlotSchw);
    ui.progressBarSM->hide();
    PlotSchwHisto = new QwtPlot(QwtText("Schwarzschild model orbit weights"), ui.frame_smarea);
    sym.setSize(2);
    sym.setStyle(QwtSymbol::Rect);
    sym.setBrush(QColor(Qt::blue));
    CurveSchwHistoR = new QwtPlotCurve("");
    CurveSchwHistoR->setStyle(QwtPlotCurve::Dots);
    CurveSchwHistoR->setSymbol(sym);
    CurveSchwHistoR->attach(PlotSchwHisto);
    sym.setBrush(QColor(Qt::red));
    CurveSchwHistoC = new QwtPlotCurve("");
    CurveSchwHistoC->setStyle(QwtPlotCurve::Dots);
    CurveSchwHistoC->setSymbol(sym);
    CurveSchwHistoC->attach(PlotSchwHisto);
    LPlotSchw->addWidget(PlotSchwHisto);
    QwtPlotPicker *PickerSchwHisto = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection, QwtPlotPicker::NoRubberBand, QwtPicker::AlwaysOn, PlotSchwHisto->canvas());
    PickerSchwHisto->setMousePattern(QwtEventPattern::MouseSelect1, Qt::RightButton);
    connect(PickerSchwHisto, SIGNAL(selected(const QwtDoublePoint &)), SLOT(SMHistoClicked(const QwtDoublePoint &)));
    PlotSchwHisto->hide();

	// connect various events to change corresponding parameters in core
    connect(ui.lineEdit_x, SIGNAL(textChanged(const QString&)), SLOT(ICchanged()));
    connect(ui.lineEdit_y, SIGNAL(textChanged(const QString&)), SLOT(ICchanged()));
    connect(ui.lineEdit_z, SIGNAL(textChanged(const QString&)), SLOT(ICchanged()));
    connect(ui.lineEdit_vx, SIGNAL(textChanged(const QString&)), SLOT(ICchanged()));
    connect(ui.lineEdit_vy, SIGNAL(textChanged(const QString&)), SLOT(ICchanged()));
    connect(ui.lineEdit_vz, SIGNAL(textChanged(const QString&)), SLOT(ICchanged()));
    connect(ui.lineEdit_E, SIGNAL(textChanged(const QString&)), SLOT(ICchanged()));
    connect(ui.lineEdit_q, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_p, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_Mbh, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_Rc, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_Gamma, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_Alpha, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_Ncoefs_radial, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_Ncoefs_angular, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_eps, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.lineEdit_tol, SIGNAL(textChanged(const QString&)), SLOT(potentialchanged()));
    connect(ui.selectPotentialType, SIGNAL(currentIndexChanged(int)), SLOT(potentialchanged()));
    connect(ui.radioButton_2dorbit, SIGNAL(toggled(bool)), SLOT(redrawOrbit(bool)));
    connect(ui.radioButton_3dline, SIGNAL(toggled(bool)), SLOT(redrawOrbit(bool)));
    connect(ui.radioButton_3dmesh, SIGNAL(toggled(bool)), SLOT(redrawOrbit(bool)));
    connect(ui.radioButton_2d_xy, SIGNAL(toggled(bool)), SLOT(redrawOrbit(bool)));
    connect(ui.radioButton_2d_xz, SIGNAL(toggled(bool)), SLOT(redrawOrbit(bool)));
    connect(ui.radioButton_2d_yz, SIGNAL(toggled(bool)), SLOT(redrawOrbit(bool)));
    connect(ui.radioButton_fmmapdiff, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmmaplyapunov, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmhistodiff, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmhistolyapunov, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmssdiff, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmsslyapunov, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmspdiff, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmsplyapunov, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmsydiff, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_fmsylyapunov, SIGNAL(toggled(bool)), SLOT(redrawFreqMap(bool)));
    connect(ui.radioButton_smviewgrid, SIGNAL(toggled(bool)), SLOT(redrawSchwModel(bool)));
    connect(ui.radioButton_smviewweights, SIGNAL(toggled(bool)), SLOT(redrawSchwModel(bool)));
    connect(ui.radioButton_smviewhisto, SIGNAL(toggled(bool)), SLOT(redrawSchwModel(bool)));
    connect(ui.lineEdit_intTime, SIGNAL(textChanged(const QString&)), SLOT(intTime_changed()));
    connect(ui.lineEdit_timeStep,SIGNAL(textChanged(const QString&)), SLOT(intTime_changed()));
    connect(ui.lineEdit_fms, SIGNAL(textChanged(const QString&)), SLOT(fmparams_changed()));
    connect(ui.lineEdit_fmp, SIGNAL(textChanged(const QString&)), SLOT(fmparams_changed()));
    connect(ui.lineEdit_fmy, SIGNAL(textChanged(const QString&)), SLOT(fmparams_changed()));
    connect(ui.lineEdit_fmr, SIGNAL(textChanged(const QString&)), SLOT(fmparams_changed()));
    connect(ui.lineEdit_sms, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smp, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smy, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smr, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smlinespersegment, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smenergylevels, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smNumBunchOrbits, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smNumSamplingPoints, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.checkBox_smSuperOrbit, SIGNAL(toggled(bool)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smchaoticweight, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smMaxShellSuperOrbit, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.checkBox_smBeta, SIGNAL(toggled(bool)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smBetaIn, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smBetaOut, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_smMaxOrbitWeight, SIGNAL(textChanged(const QString&)), SLOT(smparams_changed()));
    connect(ui.lineEdit_MinFreqDiff, SIGNAL(textChanged(const QString&)), SLOT(ccparams_changed()));
    connect(ui.lineEdit_MinLambda, SIGNAL(textChanged(const QString&)), SLOT(ccparams_changed()));
    ui.lineEdit_x->installEventFilter(this);
    ui.lineEdit_y->installEventFilter(this);
    ui.lineEdit_z->installEventFilter(this);
    ui.lineEdit_vx->installEventFilter(this);
    ui.lineEdit_vy->installEventFilter(this);
    ui.lineEdit_vz->installEventFilter(this);
    ui.lineEdit_q->installEventFilter(this);
    ui.lineEdit_p->installEventFilter(this);
    ui.lineEdit_Mbh->installEventFilter(this);
    ui.lineEdit_Rc->installEventFilter(this);
    ui.lineEdit_intTime->installEventFilter(this);
    ui.lineEdit_timeStep->installEventFilter(this);
    ui.checkBox_Lyapunov->installEventFilter(this);
    ui.radioButton_2d->installEventFilter(this);
    ui.radioButton_3d->installEventFilter(this);
    ui.radioButton_2dorbit->installEventFilter(this);
    ui.radioButton_3dline->installEventFilter(this);
    ui.radioButton_3dmesh->installEventFilter(this);
    ui.ButtonClearPS->installEventFilter(this);
    ui.ButtonRandom->installEventFilter(this);
    ui.tab_f->installEventFilter(this);
    ui.tab_p->installEventFilter(this);
    ui.tab_o->installEventFilter(this);
    ui.tabWidget->installEventFilter(this);
    PlotOrbit->installEventFilter(this);
    PlotPoincare->installEventFilter(this);
    PlotFreq->installEventFilter(this);
    PlotFreqMap->installEventFilter(this);

    // load and apply settings
    loadApplySettings();
}

CSmileGUI::~CSmileGUI()
{
    if(ui.checkBox_saveSettings->isChecked())
        core->saveSettings();
}

bool CSmileGUI::loadApplySettings(QString fileName)
{
    bool result=core->loadSettings(fileName);
    InternalRecalc=true;
    ui.selectPotentialType->setCurrentIndex((int)core->PotentialType);
    ui.lineEdit_q->setText(QString::number(core->q));
    ui.lineEdit_p->setText(QString::number(core->p));
    ui.lineEdit_Mbh->setText(QString::number(core->Mbh));
    ui.lineEdit_Rc->setText(QString::number(core->Rc));
    ui.lineEdit_Gamma->setText(QString::number(core->Gamma));
    ui.lineEdit_Ncoefs_radial->setText(QString::number(core->Ncoefs_radial));
    ui.lineEdit_Ncoefs_angular->setText(QString::number(core->Ncoefs_angular));
    ui.lineEdit_Alpha->setText(QString::number(core->Alpha));
    ui.lineEdit_NbodyFile->setText(core->NbodyFile);
    ui.lineEdit_eps->setText(QString::number(config.nbpotEps));
    ui.lineEdit_tol->setText(QString::number(config.nbpotTol));
    ui.lineEdit_x->setText(QString::number(core->icX));
    ui.lineEdit_y->setText(QString::number(core->icY));
    ui.lineEdit_z->setText(QString::number(core->icZ));
    ui.lineEdit_vx->setText(QString::number(core->icVx));
    ui.lineEdit_vy->setText(QString::number(core->icVy));
    ui.lineEdit_vz->setText(QString::number(core->icVz));
    ui.lineEdit_E->setText(QString::number(core->E));
    ui.lineEdit_intTime->setText(QString::number(config.intTimeInPeriods));
    ui.lineEdit_timeStep->setText(QString::number(config.intTimeStepsPerPeriod));
    ui.checkBox_Lyapunov->setChecked(config.calcLyapunov);
    ui.checkBox_usePS->setChecked(config.usePS);
    ui.lineEdit_fms->setText(QString::number(config.fm_numOrbitsStationary));
    ui.lineEdit_fmp->setText(QString::number(config.fm_numOrbitsPrincipalPlane));
    ui.lineEdit_fmy->setText(QString::number(config.fm_numOrbitsYalpha));
    ui.lineEdit_fmr->setText(QString::number(config.fm_numOrbitsRandom));
    ui.lineEdit_sms->setText(QString::number(config.sm_numOrbitsStationary));
    ui.lineEdit_smp->setText(QString::number(config.sm_numOrbitsPrincipalPlane));
    ui.lineEdit_smy->setText(QString::number(config.sm_numOrbitsYalpha));
    ui.lineEdit_smr->setText(QString::number(config.sm_numOrbitsRandom));
    ui.lineEdit_smlinespersegment->setText(QString::number(config.sm_linesPerSegment));
    ui.lineEdit_smenergylevels->setText(QString::number(config.sm_numShells));
    ui.lineEdit_smNumBunchOrbits->setText(QString::number(config.sm_numBunchOrbits));
    ui.lineEdit_smNumSamplingPoints->setText(QString::number(config.sm_numSamplingPoints));
    ui.selectEnergyLevel->setMaximum(config.sm_numShells);
    ui.lineEdit_MinFreqDiff->setText(QString::number(config.chaoticMinFreqDiff));
    ui.lineEdit_MinLambda->setText(QString::number(config.chaoticMinLambda));
    ui.checkBox_smSuperOrbit->setChecked(config.chaoticSuperOrbit);
    ui.lineEdit_smchaoticweight->setText(QString::number(config.chaoticWeightFactor));
    ui.lineEdit_smMaxShellSuperOrbit->setText(QString::number(config.maxShellSuperOrbit));
    ui.lineEdit_smMaxOrbitWeight->setText(QString::number(config.sm_maxWeight));
    ui.lineEdit_smBetaIn->setText(QString::number(config.sm_betaIn));
    ui.lineEdit_smBetaOut->setText(QString::number(config.sm_betaOut));
    ui.checkBox_smBeta->setChecked(config.sm_constrainBeta);
    ui.lineEdit_smBetaIn->setEnabled(config.sm_constrainBeta);
    ui.lineEdit_smBetaOut->setEnabled(config.sm_constrainBeta);
    ui.ButtonSchwQP->setEnabled(config.sm_useBPMPD);
    InternalRecalc=false;
    potentialchanged();
    if(core->useICe) ui.radioButton_ICe->setChecked(true); else ui.radioButton_ICe->setChecked(false);
    if(core->N_dim==2) ui.radioButton_2d->setChecked(true); else ui.radioButton_3d->setChecked(true);
    smparams_changed();
    core->initIC();
    ui.lineEdit_Torb->setText(QString::number(core->Torbx));
    return result;
}

bool CSmileGUI::eventFilter(QObject* target, QEvent* Event)
{
    if(Event->type() == QEvent::KeyPress) 
    {  
        QKeyEvent* keyEvent = static_cast<QKeyEvent*>(Event);
        if((keyEvent->key() == Qt::Key_Return) || (keyEvent->key() == Qt::Key_Enter))
        {
            on_ButtonStart_clicked();
            return true;
        }
    }
    return QMainWindow::eventFilter(target, Event);
}

void CSmileGUI::timerEventCore()
{
    if(FreqMapRunning)
    {
        ui.progressBarFM->setValue(core->orbits->nComplete);
        redrawFreqMap(true);
    }
    if(SchwModelRunning || ExportNbodyRunning)
        ui.progressBarSM->setValue(core->orbits->nComplete);
}

void CSmileGUI::timerEvent(QTimerEvent * Event)
{
    if(core->CalcThr!=NULL && Event->timerId()==myTimerOrbit)
    {
        ui.progressBarOrbit->setValue(core->orbit->nActual*100/core->orbit->nPoints);
        ui.progressBarOrbit->show();
        redrawOrbit(true);
    }
}

void CSmileGUI::info(const QString &message)
{
    ui.label_info->setPlainText(message);
    ui.label_info->repaint();
    // to refresh orbit weights in Lucy iterations
    if(message.startsWith("Iteration") && ui.tab_sm->isVisible())
        redrawSchwModel(false);

}

/// ----------------------------  single orbit integration --------- ///
void CSmileGUI::on_ButtonStart_clicked()
{
    if(core->CalcThr!=NULL) return;
    if(core->Torbx<0) return;
    ui.groupBox_N_dim->setEnabled(false);
    ui.groupBox_potential->setEnabled(false);
    ui.ButtonStart->setEnabled(false);
    ui.ButtonRandom->setEnabled(false);
    core->startOrbit();
    timeOI.start();
    myTimerOrbit = startTimer(2000);  // to refresh orbit plot
}

void CSmileGUI::CalcFinished()    // called from core->coreOrbitFinished()
{
    ui.groupBox_N_dim->setEnabled(true);
    ui.groupBox_potential->setEnabled(true);
    ui.ButtonStart->setEnabled(true);
    ui.ButtonRandom->setEnabled(true);
    timeOIelapsed=timeOI.elapsed();  // store to re-use in subsequent "refresh spectrum" events
    killTimer(myTimerOrbit);
    ui.progressBarOrbit->hide();
    // poincare
	if(core->orbit->usePS && core->orbit->ps.size()>0)
	{
		QwtPlotCurve* CurvePS = new QwtPlotCurve();
		QwtSymbol sym;
		sym.setStyle(QwtSymbol::Rect);
        sym.setPen(Qt::NoPen);
        sym.setBrush(QColor(rand()%224, rand()%192, rand()%224));
		sym.setSize(2);
		CurvePS->setSymbol(sym);
		CurvePS->setStyle(QwtPlotCurve::Dots);
		CurvePS->attach(PlotPoincare);
		int nPS=core->orbit->ps.size();
		vectord psx(nPS), psvx(nPS);
		for(int i=0; i<nPS; i++)
		{
			psx[i]=core->orbit->ps[i].first;
			psvx[i]=core->orbit->ps[i].second;
		}
        // remove closest
        bool actiontaken;
        if(psx.size()>100)
        {
            double maxdistx=0, maxdistvx=0;
            for(int k1=1; k1<psx.size(); k1++)
            {
                double x1=psx[k1], y1=psvx[k1];
                for(int k2=0; k2<k1; k2++)
                {
                    double d=fabs(psx[k2]-x1);
                    if(d>maxdistx)
                        maxdistx=d;
                    d=fabs(psvx[k2]-y1);
                    if(d>maxdistvx) maxdistvx=d;
                }
            }
          do{
            double mindist=1000;
            int minp1, minp2;
            for(int k1=1; k1<psx.size(); k1++)
            {
                double x1=psx[k1], y1=psvx[k1];
                for(int k2=0; k2<k1; k2++)
                {
                    double d=fabs(psx[k2]-x1)/maxdistx + fabs(psvx[k2]-y1)/maxdistvx;
                    if(d<mindist)
                    {
                        mindist=d;
                        minp1=k1;
                        minp2=k2;
                    }
                }
            }
            if(mindist<1e-3)
            {
                psx.erase(psx.begin()+minp1);
                psvx.erase(psvx.begin()+minp1);
                actiontaken=true;
            }
            else
                actiontaken=false;
          }while(actiontaken);
        }
		CurvePS->setData(&(psx.front()), &(psvx.front()), psx.size());
		PlotPoincare->replot();
		CurvesPoincare<<(CurvePS);
	}
    // perform frequency graphs and orbit classification (also set label text and redraw orbit)
    if(ui.spinBox_intervalCount->value()!=1)
        ui.spinBox_intervalCount->setValue(1);  // refreshes frequency graph and calls for orbit classification
    else
        on_spinBox_intervalCount_valueChanged(1);
    // lyapunov
    if(core->orbit->calcLyapunov)
      if(core->orbit->nActual<100)
      {
        CurveLyapunov->setData(NULL, NULL, 0);
        CurveDeviation->setData(NULL, NULL, 0);
        PlotLyapunov->replot();
      }
      else
      {
        vectord timesteps(core->orbit->nActual);
        vectord lambdatmp(core->orbit->nActual);
        double devvecmin=1e4;
        for(int s=100; s<core->orbit->nActual; s++) 
        {
            timesteps[s] = s*core->orbit->timeStep/core->orbit->timeUnit;
            lambdatmp[s] = std::max<double>((core->orbit->lnw[s])/(s*core->orbit->timeStep/core->orbit->timeUnit), 1e-4);
            devvecmin = std::min<double>(core->orbit->lnw[s] - log(s*core->orbit->timeStep/core->orbit->timeUnit), devvecmin);
        }
        double devvecadd=(devvecmin<0.1?0.1-devvecmin:0);
        CurveLyapunov->setData(&(timesteps.at(100)), &(lambdatmp.at(100)), core->orbit->nActual-100);
        for(int s=100; s<core->orbit->nActual; s++) 
            lambdatmp[s] = core->orbit->lnw[s] - log(s*core->orbit->timeStep/core->orbit->timeUnit) + devvecadd;
        CurveDeviation->setData(&(timesteps.at(100)), &(lambdatmp.at(100)), core->orbit->nActual-100);
        //for(int i=0; i<PlotLyapunov->axisCnt; i++) PlotLyapunov->setAxisAutoScale(i); 
        PlotLyapunov->setAxisScale(QwtPlot::yLeft, 1e-4, 10);
        PlotLyapunov->setAxisScale(QwtPlot::yRight, 0.1, 1e4);
        PlotLyapunov->setAxisScale(QwtPlot::xBottom, 1, core->orbit->intTime/core->orbit->timeUnit*1.1);
        PlotLyapunov->replot();
      }
}

/// ----------------------------  orbit rendering --------- ///
void CSmileGUI::redrawOrbit(bool show)
{
    if(!show || !core->orbit || !core->orbit->tp[0].size()) return;
    int intNum = ui.spinBox_IntervalNum->value()-1;
    int intCount = ui.spinBox_intervalCount->value();
    int start=core->orbit->nActual * intNum / intCount;
    int count=core->orbit->nActual / intCount; 
    if(start+count>core->orbit->nActual) count=core->orbit->nActual-start;
    ui.groupBox_2dplane->setEnabled(true);
    if(ui.radioButton_2dorbit->isChecked())
    {   // draw 2d orbit projection
        double* hor, *ver;
        if(core->orbit->N_dim==2)
        {
            hor=&(core->orbit->tp[0].at(start)); ver=&(core->orbit->tp[1].at(start)); 
        }
        else
        {
            if(ui.radioButton_2d_yz->isChecked()) { hor=&(core->orbit->tp[1].at(start)); ver=&(core->orbit->tp[2].at(start)); }
            if(ui.radioButton_2d_xz->isChecked()) { hor=&(core->orbit->tp[0].at(start)); ver=&(core->orbit->tp[2].at(start)); }
            if(ui.radioButton_2d_xy->isChecked()) { hor=&(core->orbit->tp[0].at(start)); ver=&(core->orbit->tp[1].at(start)); }
        }
        CurveOrbit->setData(hor, ver, count);
        // equipotential surface
        double eqh[NUM_POINTS_PLOT_EQUIPOTENTIAL*4], eqv[NUM_POINTS_PLOT_EQUIPOTENTIAL*4];
        double E = core->E;
        int num_points_plot_equipotential = (core->potential->PotentialType()==CPotential::PT_NB)?NUM_POINTS_PLOT_EQUIPOTENTIAL/5:NUM_POINTS_PLOT_EQUIPOTENTIAL;  
        for(int i=0; i<num_points_plot_equipotential; i++)
        {
            double angle=(i+0.5)*M_PI/2/num_points_plot_equipotential;
            double X, Y, Z, h, v;
            if(ui.radioButton_2d_xy->isChecked()) { X=cos(angle); Y=sin(angle); Z=0; h=X; v=Y; }
            if(ui.radioButton_2d_yz->isChecked()) { Y=cos(angle); Z=sin(angle); X=0; h=Y; v=Z; }
            if(ui.radioButton_2d_xz->isChecked()) { X=cos(angle); Z=sin(angle); Y=0; h=X; v=Z; }
            double r=core->potential->findintersection(E, X, Y, Z);
            eqh[i]=h*r; 
            eqv[i]=v*r;
            eqh[num_points_plot_equipotential*2-1-i]=-h*r;
            eqv[num_points_plot_equipotential*2-1-i]=v*r;
            eqh[num_points_plot_equipotential*2+i]=-h*r;
            eqv[num_points_plot_equipotential*2+i]=-v*r;
            eqh[num_points_plot_equipotential*4-1-i]=h*r;
            eqv[num_points_plot_equipotential*4-1-i]=-v*r;
        }
        CurveEquipotential->setData(eqh, eqv, 4*num_points_plot_equipotential);
        double scale = std::max<double>(sqrt(sqr(eqh[num_points_plot_equipotential/2])+sqr(eqv[num_points_plot_equipotential/2]))/1.3, core->orbit->maxr);
        PlotOrbit->setAxisScale(QwtPlot::yLeft, -scale, scale);
        PlotOrbit->setAxisScale(QwtPlot::xBottom, -scale, scale);
        PlotOrbit->replot();
#ifdef USEQWT3D
        Plot3d->hide();
#endif
        PlotOrbitParam->hide();
        PlotOrbit->show();
    }
    else if(ui.radioButton_3dline->isChecked() || ui.radioButton_3dmesh->isChecked())
    {   // draw 3d orbit
        ui.groupBox_2dplane->setEnabled(false);
        ui.groupBox_3dmesh->setEnabled(false);
#ifdef USEQWT3D 
        if(ui.radioButton_3dline->isChecked())
        {
            Cell trjcell;
            trjpos.clear();
            trjcells.clear();
            trjcell.clear();
            vectord op[N_DIM];
            for (int i=start; i<start+count; i++)
            {
                trjpos.push_back(Triple(core->orbit->tp[0][i], core->orbit->tp[1][i], core->orbit->tp[2][i]));
                trjcell.push_back(i-start);
            }
            trjcells.push_back(trjcell);
            // draw trajectory
            Plot3d->loadFromData(trjpos, trjcells);
            Plot3d->setPlotStyle(Qwt3D::WIREFRAME);
        }
#ifdef USE3DFACETS
        if(ui.radioButton_3dmesh->isChecked())
        {
            // draw mesh
            if(TriangThr->restart)   // triangulation is in process, so do not display anything
                Plot3d->loadFromData(TripleField(), CellField());
            else
                Plot3d->loadFromData(TriangThr->trjpos, TriangThr->facets);
            Plot3d->setPlotStyle(Qwt3D::FILLEDMESH);
            ui.groupBox_3dmesh->setEnabled(true);
        }
#endif
        Plot3d->updateData();
        Plot3d->updateGL();
        Plot3d->show();
        PlotOrbit->hide();
#endif
    }
}

void CSmileGUI::on_pushButton_refresh3dmesh_clicked()
{
#ifdef USE3DFACETS
    bool ok;
    double md = ui.lineEdit_3dmesh_maxdist->text().toDouble(&ok);   
    if(ok) core->maxdist=md;
    int meshstyle=0;
    if(ui.radioButton_3dmesh_x->isChecked()) meshstyle=1; else
    if(ui.radioButton_3dmesh_y->isChecked()) meshstyle=2; else
    if(ui.radioButton_3dmesh_z->isChecked()) meshstyle=3; 
    TriangThr->startTriangulation(core->orbit->tp, core->maxdist, meshstyle);
#endif
}
/// ----------------------------  spectrum rendering --------- ///
void CSmileGUI::on_spinBox_intervalCount_valueChanged(int count)
{
    if(InternalRecalc) return;
    bool doNotRefresh = (ui.spinBox_IntervalNum->value() > count);
    ui.spinBox_IntervalNum->setMaximum(count);
    if(doNotRefresh) return;   // setting maximum more than current value led to change of value of IntervalNum and hence the refresh function was already called
    if(ui.spinBox_IntervalNum->value()!=1)
        ui.spinBox_IntervalNum->setValue(1);  // refreshes frequency graph and calls for orbit classification
    else
        refreshOrbitAndSpectrum();
}

void CSmileGUI::on_spinBox_IntervalNum_valueChanged(int)
{
    refreshOrbitAndSpectrum();
}

void CSmileGUI::refreshOrbitAndSpectrum() 
{
    int intNum = ui.spinBox_IntervalNum->value() - 1;
    int intCount = ui.spinBox_intervalCount->value();
    if (intNum>=intCount) return;
    int start=core->orbit->nActual * intNum / intCount;
    int count=core->orbit->nActual / intCount;

    timeOI.start();
    // obtain spectrum for given interval
    Spectrum sl[N_DIM];
    std::vector<double> fa[N_DIM];
    core->orbit->performFrequencyAnalysis(sl, fa, start, count);
    int nPointsSpectrum = fa[0].size();   // should be count/2+1, but sometimes "count--;" may happen (to speed up FFT in the case of "count" being large prime)
    vectord freq(nPointsSpectrum);
    for(int i=0; i<nPointsSpectrum; i++)
        freq[i]=i/core->orbit->intTime*core->orbit->timeUnit * intCount;
    CurveFreqX->setData(&freq.front(), &fa[0].front(), nPointsSpectrum*std::min<double>(2*PLOT_FREQ_MAX_F/ui.lineEdit_timeStep->text().toDouble(),1));
    CurveFreqY->setData(&freq.front(), &fa[1].front(), nPointsSpectrum*std::min<double>(2*PLOT_FREQ_MAX_F/ui.lineEdit_timeStep->text().toDouble(),1));
    if(core->N_dim==3)
        CurveFreqZ->setData(&freq.front(), &fa[2].front(), nPointsSpectrum*std::min<double>(2*PLOT_FREQ_MAX_F/ui.lineEdit_timeStep->text().toDouble(),1));
    else
        CurveFreqZ->setData(NULL, NULL, 0);
    double sf[MAX_SPECTRAL_LINES*N_DIM], sa[MAX_SPECTRAL_LINES*N_DIM];
    double sfp[MAX_SPECTRAL_LINES*N_DIM], sap[MAX_SPECTRAL_LINES*N_DIM];
    int si=0, sip=0;
    for(int d=0; d<core->N_dim; d++)
      for(int s=0; s<sl[d].size() && s<MAX_SPECTRAL_LINES*N_DIM; s++) 
          if(sl[d][s].freq<5)
          {
            if(sl[d][s].precise)
            {
                sfp[sip]=sl[d][s].freq;
                sap[sip]=sl[d][s].ampl;
                sip++;
            }
            else
            {
                sf[si]=sl[d][s].freq;
                sa[si]=sl[d][s].ampl;
                si++;
            }
          }
    CurveFreqPeaks->setData(sf, sa, si);
    CurveFreqPeaksP->setData(sfp, sap, sip);
    for(int i=0; i<PlotFreq->axisCnt; i++) PlotFreq->setAxisAutoScale(i); 
    PlotFreq->replot();
    ZoomerFreq->setZoomBase(true);
    double inertia[N_DIM];
    core->orbit->calcInertiaOverInterval(start, count, inertia);

#ifdef USEQWT3D
#ifdef USE3DFACETS
    if(core->N_dim==3)
    {
        ui.radioButton_3dmesh_full->setChecked(true);
        ui.lineEdit_3dmesh_maxdist->setText(QString::number(core->maxdist, 'g', 3));
        TriangThr->startTriangulation(core->orbit->tp, core->maxdist, 0);
    }
#endif
#endif
    // redraw orbit
    redrawOrbit(true);

    // set label text
    ui.label_info->setPlainText("lfX="+QString::number(sl[0][0],'f',7) + "\nlfY="+QString::number(sl[1][0],'f',7) +
        ((core->N_dim==3)?"\nlfZ="+QString::number(sl[2][0],'f',7):"") +
        "\n"+core->orbit->Description + 
        "\nFreqDiff="+QString::number(core->orbit->calcLFCCdiffusion(start, count),'g',3)+" ("+core->orbit->DescriptionLFCCdiff+")" +
        (core->orbit->calcLyapunov?("\nLyapunovExp="+QString::number(core->orbit->calcLambda(),'g',3)):"") +
        "\nInertia: "+QString::number((inertia[0]),'g',3)+", "+QString::number((inertia[1]),'g',3)+", "+QString::number((inertia[2]),'g',3) +
        "\nLz="+QString::number(core->orbit->amz,'g',3)+""+QString::number(sqrt(sqr(core->orbit->amzrms)-sqr(core->orbit->amz)),'g',3) + 
        (core->N_dim==3 ? "\nLx="+QString::number(core->orbit->amx,'g',3)+""+QString::number(sqrt(sqr(core->orbit->amxrms)-sqr(core->orbit->amx)),'g',3) : "") + 
        "\nEdiff="+QString::number(core->orbit->Edif) +
        "\nRmin="+QString::number(core->orbit->minr) + 
        "\nTime elapsed="+QString::number((timeOIelapsed+timeOI.elapsed())*0.001)+" s"
        );
    ui.label_info->repaint();    
}

void CSmileGUI::on_checkBox_freqLines_stateChanged(int state)
{
    if(ui.checkBox_freqLines->isChecked())
    {
        CurveFreqPeaksP->setStyle(QwtPlotCurve::Sticks);
        CurveFreqPeaks->setStyle(QwtPlotCurve::Sticks);
    }
    else
    {
        CurveFreqPeaksP->setStyle(QwtPlotCurve::NoCurve);
        CurveFreqPeaks->setStyle(QwtPlotCurve::NoCurve);
    }
    PlotFreq->replot();   
}

/// ----------------------------  update initial conditions --------- ///
void CSmileGUI::potentialchanged()
{
    if(InternalRecalc) return;
    CPotential::POTENTIALTYPE PotentialType = (CPotential::POTENTIALTYPE)ui.selectPotentialType->currentIndex();
    if(PotentialType==CPotential::PT_BSE)
    {
        ui.lineEdit_Alpha->show();
        ui.label_Alpha->show();
    }
    else
    {
        ui.lineEdit_Alpha->hide();
        ui.label_Alpha->hide();
    }
    if(PotentialType==CPotential::PT_DEHNENBSE || PotentialType==CPotential::PT_PLUMMERBSE || PotentialType==CPotential::PT_BSE)
    {
        ui.lineEdit_Ncoefs_radial->show();
        ui.label_Ncoefs_radial->show();
    }
    else
    {
        ui.lineEdit_Ncoefs_radial->hide();
        ui.label_Ncoefs_radial->hide();
    }
    if(PotentialType==CPotential::PT_DEHNENBSE || PotentialType==CPotential::PT_PLUMMERBSE || PotentialType==CPotential::PT_BSE || PotentialType==CPotential::PT_SCALEFREEBSE)
    {
        ui.lineEdit_Ncoefs_angular->show();
        ui.label_Ncoefs_angular->show();
    }
    else
    {
        ui.lineEdit_Ncoefs_angular->hide();
        ui.label_Ncoefs_angular->hide();
    }
    if(PotentialType==CPotential::PT_BSE || PotentialType==CPotential::PT_NB)
    {
        ui.lineEdit_NbodyFile->show();
        ui.Button_NbodyFile->show();
    }
    else
    {
        ui.lineEdit_NbodyFile->hide();
        ui.Button_NbodyFile->hide();
    }
    if(PotentialType==CPotential::PT_LOG)
    {
        ui.lineEdit_Rc->show();
        ui.label_core->show();
    }
    else
    {
        ui.lineEdit_Rc->hide();
        ui.label_core->hide();
    }
    if( PotentialType==CPotential::PT_DEHNEN || PotentialType==CPotential::PT_DEHNENBSE || 
        PotentialType==CPotential::PT_SCALEFREE || PotentialType==CPotential::PT_SCALEFREEBSE ||
        PotentialType==CPotential::PT_BSE || PotentialType==CPotential::PT_NB)
    {
        ui.lineEdit_Gamma->show();
        ui.label_cusp->show();
    }
    else
    {
        ui.lineEdit_Gamma->hide();
        ui.label_cusp->hide();
    }
    if(PotentialType==CPotential::PT_NB)
    {
        ui.label_eps->show();
        ui.label_tol->show();
        ui.lineEdit_eps->show();
        ui.lineEdit_tol->show();
        // set N_dim=3
        core->N_dim=3;
        bool IntR=InternalRecalc;
        InternalRecalc=true;
        on_radioButton_2d_toggled(false);
        InternalRecalc=IntR;
        ui.radioButton_3d->setChecked(true);
        ui.groupBox_2d3d->setEnabled(false);
        // disable Lyapunov exponent
        config.calcLyapunov=false;
        ui.checkBox_Lyapunov->setChecked(false);
        ui.checkBox_Lyapunov->setEnabled(false);
    }
    else
    {
        ui.label_eps->hide();
        ui.label_tol->hide();
        ui.lineEdit_eps->hide();
        ui.lineEdit_tol->hide();
        ui.groupBox_2d3d->setEnabled(true);
        ui.checkBox_Lyapunov->setEnabled(true);
    }

    bool ok=true;
    double newq=(ui.lineEdit_q->text().toDouble(&ok));  if(!ok || (fabs(newq)<0.1)) return;
    double newp=(ui.lineEdit_p->text().toDouble(&ok));  if(!ok || (fabs(newp)<0.1)) return;
    double newRc=ui.lineEdit_Rc->text().toDouble(&ok);  if (!ok) return;
    double newGamma=ui.lineEdit_Gamma->text().toDouble(&ok);  if (!ok) return;
    double newAlpha=ui.lineEdit_Alpha->text().toDouble(&ok);  if (!ok) return;
    double newMbh=ui.lineEdit_Mbh->text().toDouble(&ok);  if(!ok) return;
    double newNcoefs_radial=ui.lineEdit_Ncoefs_radial->text().toDouble(&ok);  if(!ok) return;
    double newNcoefs_angular=ui.lineEdit_Ncoefs_angular->text().toDouble(&ok);  if(!ok) return;
    double newnbpotEps=ui.lineEdit_eps->text().toDouble(&ok); if(!ok) return;
    double newnbpotTol=ui.lineEdit_tol->text().toDouble(&ok); if(!ok) return;
    if(PotentialType!=core->PotentialType || newq!=core->q || newp!=core->p || newRc!=core->Rc || 
        newGamma!=core->Gamma || newMbh!=core->Mbh || newNcoefs_radial!=core->Ncoefs_radial || newNcoefs_angular!=core->Ncoefs_angular || 
        newAlpha!=core->Alpha || newnbpotEps!=config.nbpotEps || newnbpotTol!=config.nbpotTol)
    {
        core->q=newq; core->p=newp; core->Rc=newRc; core->Gamma=newGamma; core->Mbh=newMbh; 
        core->Ncoefs_radial=newNcoefs_radial; core->Ncoefs_angular=newNcoefs_angular; core->Alpha=newAlpha;
        config.nbpotEps=newnbpotEps; config.nbpotTol=newnbpotTol;
        core->initPotential(PotentialType);
        ICchanged();
    }
    ui.frame->repaint();
}

void CSmileGUI::ICchanged()
{
    if(InternalRecalc) return;
    bool ok=true;
    double newE=ui.lineEdit_E->text().toDouble(&ok);  if(!ok) return;
    double X=ui.lineEdit_x->text().toDouble(&ok);  if(!ok) return;
    double Y=ui.lineEdit_y->text().toDouble(&ok);  if(!ok) return;
    double Z=(core->N_dim==3)?ui.lineEdit_z->text().toDouble(&ok):0;  if(!ok) return;
    double Vx=ui.lineEdit_vx->text().toDouble(&ok);  if(!ok) return;
    double Vy=ui.lineEdit_vy->text().toDouble(&ok);  if(!ok) return;
    double Vz=(core->N_dim==3)?ui.lineEdit_vz->text().toDouble(&ok):0;  if(!ok) return;
    if(core->useICe)
    {
        core->E=newE;
    }
    else
    {
        core->icX=X; core->icY=Y; core->icZ=Z; core->icVx=Vx; core->icVy=Vy; core->icVz=Vz;
    }
    InternalRecalc=true;
    bool changed = core->initIC();
    if(core->useICe || changed)
    {
        ui.lineEdit_x->setText(QString::number(core->icX));
        ui.lineEdit_y->setText(QString::number(core->icY));
        ui.lineEdit_z->setText(QString::number(core->icZ));
        ui.lineEdit_vx->setText(QString::number(core->icVx));
        ui.lineEdit_vy->setText(QString::number(core->icVy));
        ui.lineEdit_vz->setText(QString::number(core->icVz));
        if(changed)
            ui.lineEdit_E->setText(QString::number(core->E));
    }
    else
    {
        ui.lineEdit_E->setText(QString::number(core->E));
    }
    ui.lineEdit_Torb->setText(QString::number(core->Torbx));
    //info("density="+QString::number(core->potential->Rho(core->icX, core->icY, core->icZ)));
    InternalRecalc=false;
    if(fabs(core->E-Eprev)> 1e-4)
        on_ButtonClearPS_clicked();
}

/// ----------------------------  modification of various parameters by GUI lineEdits ----- ///
void CSmileGUI::intTime_changed()
{
    if(InternalRecalc) return;
    bool ok;
    double t = ui.lineEdit_intTime->text().toDouble(&ok);
    if(ok)  config.intTimeInPeriods=t;
    t = ui.lineEdit_timeStep->text().toDouble(&ok);
    if(ok)  config.intTimeStepsPerPeriod=t;
}

void CSmileGUI::fmparams_changed()
{
    if(InternalRecalc) return;
    bool ok; int t;
    t = ui.lineEdit_fms->text().toInt(&ok);
    if(ok)  config.fm_numOrbitsStationary=t;
    t = ui.lineEdit_fmp->text().toInt(&ok);
    if(ok)  config.fm_numOrbitsPrincipalPlane=t;
    t = ui.lineEdit_fmy->text().toInt(&ok);
    if(ok)  config.fm_numOrbitsYalpha=t;
    t = ui.lineEdit_fmr->text().toInt(&ok);
    if(ok)  config.fm_numOrbitsRandom=t;
}

void CSmileGUI::smparams_changed()
{
    if(InternalRecalc) return;
    bool ok; int t; double d;
    t = ui.lineEdit_sms->text().toInt(&ok);
    if(ok)  config.sm_numOrbitsStationary=t;
    t = ui.lineEdit_smp->text().toInt(&ok);
    if(ok)  config.sm_numOrbitsPrincipalPlane=t;
    t = ui.lineEdit_smy->text().toInt(&ok);
    if(ok)  config.sm_numOrbitsYalpha=t;
    t = ui.lineEdit_smr->text().toInt(&ok);
    if(ok)  config.sm_numOrbitsRandom=t;
    t = ui.lineEdit_smlinespersegment->text().toInt(&ok);
    if(ok&&(config.sm_linesPerSegment!=t))
    {
        ui.groupBox_smOptim->setEnabled(false);
        config.sm_linesPerSegment=t;
    }
    t = ui.lineEdit_smenergylevels->text().toInt(&ok);
    if(ok)
    {
        if(config.sm_numShells!=t) ui.groupBox_smOptim->setEnabled(false);
        ui.selectEnergyLevel->setMaximum(t);
        ui.selectEnergyLevel_fm->setMaximum(t);
        config.sm_numShells=t;
    }
    t = ui.lineEdit_smNumBunchOrbits->text().toInt(&ok);
    if(ok&&(config.sm_numBunchOrbits!=t))
    {
        ui.groupBox_smOptim->setEnabled(false);
        config.sm_numBunchOrbits=t;
    }
    t = ui.lineEdit_smNumSamplingPoints->text().toInt(&ok);
    if(ok&&(config.sm_numSamplingPoints!=t))
    {
        ui.groupBox_smOptim->setEnabled(false);
        config.sm_numSamplingPoints=t;
    }
    ui.lineEdit_smgridcells->setText(QString::number(3*config.sm_linesPerSegment*config.sm_linesPerSegment*config.sm_numShells));
    config.chaoticSuperOrbit = ui.checkBox_smSuperOrbit->isChecked();
    ui.lineEdit_smMaxShellSuperOrbit->setEnabled(config.chaoticSuperOrbit);
    t = ui.lineEdit_smMaxShellSuperOrbit->text().toInt(&ok);
    if(ok) config.maxShellSuperOrbit=t;
    d = ui.lineEdit_smchaoticweight->text().toDouble(&ok);
    if(ok) config.chaoticWeightFactor = d;
    d = ui.lineEdit_smMaxOrbitWeight->text().toDouble(&ok);
    if(ok) config.sm_maxWeight = d;
    config.sm_constrainBeta=ui.checkBox_smBeta->isChecked();
    ui.lineEdit_smBetaIn->setEnabled(config.sm_constrainBeta);
    ui.lineEdit_smBetaOut->setEnabled(config.sm_constrainBeta);
    d = ui.lineEdit_smBetaIn->text().toDouble(&ok);
    if(ok)  config.sm_betaIn = d;
    d = ui.lineEdit_smBetaOut->text().toDouble(&ok);
    if(ok)  config.sm_betaOut = d;
}

void CSmileGUI::ccparams_changed()
{
    if(InternalRecalc) return;
    bool ok;
    double d = ui.lineEdit_MinFreqDiff->text().toDouble(&ok);
    if(ok)  config.chaoticMinFreqDiff=d;
    d = ui.lineEdit_MinLambda->text().toDouble(&ok);
    if(ok)  config.chaoticMinLambda=d;
    if(core->orbits!=NULL)
        core->orbits->markChaoticOrbits();
    redrawFreqMap(true);
}

void CSmileGUI::on_checkBox_Lyapunov_stateChanged(int Checked)
{
    if(InternalRecalc) return;
    config.calcLyapunov = (Checked>0);
}

void CSmileGUI::on_checkBox_usePS_stateChanged(int Checked)
{
    if(InternalRecalc) return;
    config.usePS = (Checked>0);
}

void CSmileGUI::on_radioButton_2d_toggled(bool n2d)
{
    if(n2d)
    {
        ui.radioButton_2d_xy->setChecked(true);
        ui.radioButton_2d_xz->setEnabled(false);
        ui.radioButton_2d_yz->setEnabled(false);
        ui.radioButton_3dline->setEnabled(false);
        ui.radioButton_3dmesh->setEnabled(false);
    }
    else
    {
        ui.radioButton_2d_xz->setEnabled(true);
        ui.radioButton_2d_yz->setEnabled(true);
#ifdef USEQWT3D
        ui.radioButton_3dline->setEnabled(true);
#ifdef USE3DFACETS
        ui.radioButton_3dmesh->setEnabled(true);
#endif
#endif
    }
    if(InternalRecalc) return;
    core->N_dim=n2d?2:3;
    core->initPotential(core->PotentialType);
    ICchanged();
}

void CSmileGUI::on_radioButton_ICe_toggled(bool useICe)
{
    if(InternalRecalc) return;
    core->useICe=useICe;
    if(useICe)
    {
        ui.lineEdit_E->setEnabled(true);
        ui.groupBox_ICx->setEnabled(false);
    }
    else
    {
        ui.lineEdit_E->setEnabled(false);
        ui.groupBox_ICx->setEnabled(true);
    }
}

void CSmileGUI::on_Button_NbodyFile_clicked()
{
    QString fileName=QFileDialog::getOpenFileName(this, "Open Nbody file", core->WorkDir);
    if(fileName.isEmpty()) return;
    core->WorkDir = core->getDirName(fileName);
    core->NbodyFile=fileName;
    ui.lineEdit_NbodyFile->setText(fileName);
    core->initPotential(core->PotentialType);
    ICchanged();
}

/// ----------------------------  Random initial conditions --------- ///
void CSmileGUI::on_ButtonRandom_clicked()
{
    if(core->Xmax<0) return; // error
    const int npsampling=1000;
    std::vector<double> Xs(npsampling), Ys(npsampling), Zs(npsampling), Ws(npsampling);
    double X, Y, Z, Et, totalWeight=0;
    for(int s=0; s<npsampling; s++)
    {   // sample accessible configuration space with appropriate weight
      do{
        X = (rand()*2.0/RAND_MAX-1) * core->Xmax;
        Y = (rand()*2.0/RAND_MAX-1) * core->Xmax;
        Z = (rand()*2.0/RAND_MAX-1) * core->Xmax * (core->N_dim-2);
        Et = core->potential->Phi(X, Y, Z);
      }
      while(Et>core->E);
      double W=sqrt(2*(core->E-Et));
      Xs[s]=X; Ys[s]=Y; Zs[s]=Z; Ws[s]=W; totalWeight+=W;
    }
    // now select point from sampled
    double indw=rand() * totalWeight / RAND_MAX;
    int i=0;
    while(i<npsampling-1 && indw>=Ws[i]) 
    {
        indw-=Ws[i];
        i++;
    }
    X=Xs[i]; Y=Ys[i]; Z=Zs[i]; 
    double V=Ws[i];
    double costh=(core->N_dim==3)? (rand()*2.0/RAND_MAX-1) : 0;
    double sinth=sqrt(1-sqr(costh));
    double phi=rand()*2*M_PI/RAND_MAX;
    InternalRecalc=true;
    core->icX=X; core->icY=Y; core->icZ=Z;
    core->icVx=V*sinth*cos(phi); core->icVy=V*sinth*sin(phi); core->icVz=V*costh;
    //core->initIC();
    ui.lineEdit_x->setText(QString::number(core->icX, 'f', 7));
    ui.lineEdit_y->setText(QString::number(core->icY, 'f', 7));
    ui.lineEdit_z->setText(QString::number(core->icZ, 'f', 7));
    ui.lineEdit_vx->setText(QString::number(core->icVx, 'f', 7));
    ui.lineEdit_vy->setText(QString::number(core->icVy, 'f', 7));
    ui.lineEdit_vz->setText(QString::number(core->icVz, 'f', 7));
    InternalRecalc=false;
    on_ButtonStart_clicked();
}

/// ----------------------------  Poincare section operations --------- ///
void CSmileGUI::PSclicked(const QPointF &pos)
{
    double newx=pos.x();
    double newvx=pos.y();
    double newvy2=2*(core->E - core->potential->Phi(newx, 0, 0)) - sqr(newvx);
    if(newvy2>=0)
    {
        InternalRecalc=true;
        ui.lineEdit_x->setText(QString::number(newx, 'f', 7));
        ui.lineEdit_vx->setText(QString::number(newvx, 'f', 7));
        ui.lineEdit_vy->setText(QString::number(sqrt(newvy2), 'f', 7));
        ui.lineEdit_y->setText("0");
        ui.lineEdit_z->setText("0");
        ui.lineEdit_vz->setText("0");
        InternalRecalc=false;
        ICchanged();
    }
}

void CSmileGUI::on_ButtonClearPS_clicked()
{
    while(CurvesPoincare.size()>0)
    {
        delete CurvesPoincare[CurvesPoincare.size()-1];
        CurvesPoincare.remove(CurvesPoincare.size()-1);
    }
    double xm[NUM_POINTS_PLOT_POINCARE*4], vxm[NUM_POINTS_PLOT_POINCARE*4];
    for(int i=0; i<NUM_POINTS_PLOT_POINCARE; i++)
    {
        xm[i]=core->Xmax*(i+0.5)/NUM_POINTS_PLOT_POINCARE;
        double vx2=2*(core->E - core->potential->Phi(xm[i], 0, 0));
        if(vx2>0) vxm[i]=sqrt(vx2); else vxm[i]=0;
        xm[NUM_POINTS_PLOT_POINCARE*2-1-i]=xm[i];
        vxm[NUM_POINTS_PLOT_POINCARE*2-1-i]=-vxm[i];
        xm[NUM_POINTS_PLOT_POINCARE*2+i]=-xm[i];
        vxm[NUM_POINTS_PLOT_POINCARE*2+i]=-vxm[i];
        xm[NUM_POINTS_PLOT_POINCARE*4-1-i]=-xm[i];
        vxm[NUM_POINTS_PLOT_POINCARE*4-1-i]=vxm[i];
    }
    QwtPlotCurve* CurveBoundary = new QwtPlotCurve();
    CurveBoundary->setPen(QColor(Qt::red));
    CurveBoundary->attach(PlotPoincare);
    CurveBoundary->setData(xm, vxm, 4*NUM_POINTS_PLOT_POINCARE);
    for(int i=0; i<PlotPoincare->axisCnt; i++) PlotPoincare->setAxisAutoScale(i); 
    PlotPoincare->replot();
    ZoomerPoincare->setZoomBase(true);
    CurvesPoincare<<(CurveBoundary);
    Eprev=core->E;
}

/// ----------------------------  Frequency map operations --------- ///
void CSmileGUI::on_ButtonFreqMap_clicked()
{
    if(FreqMapRunning)
    {
        emit KillFreqMapThreads();
        return;
    }
    if(SchwModelRunning || ExportNbodyRunning || core->SchwThr!=NULL) return;
    CurveFreqMapR->setData(NULL, NULL, 0);
    CurveFreqMapC->setData(NULL, NULL, 0);
    // start building freq map
    core->BuildFreqMapIC(core->E, ui.checkBox_fmExistingSS->isChecked());
    core->StartOrbitLibrary(SLOT(coreFreqMapFinished()));

    ui.progressBarFM->setRange(0, core->orbits->OrbitList.size());
    ui.progressBarFM->setValue(0);
    ui.progressBarFM->show();
    timeFM.start();
    ui.ButtonFreqMap->setText("Stop");
    ui.groupBox_N_dim->setEnabled(false);
    ui.groupBox_potential->setEnabled(false);
    for(int t=0; t<core->CalcManyThr.size(); t++)
        connect(this, SIGNAL(KillFreqMapThreads()), core->CalcManyThr[t], SLOT(stopThread()));
    FreqMapRunning=true;
}

void CSmileGUI::FreqMapFinished()
{
    FreqMapRunning=false;
    ui.ButtonFreqMap->setText("Start");
    ui.progressBarFM->hide();
    redrawFreqMap(true);
    ui.label_info->setPlainText(ui.label_info->toPlainText() + "\n"+QString::number(core->orbits->nComplete)+" orbits, " +
        QString::number(core->orbits->nComplete*1000.0/timeFM.elapsed(),'g',2)+" orbits/second\n" );
    ui.label_info->setToolTip("");
    ui.label_info->repaint();
    ui.groupBox_N_dim->setEnabled(true);
    ui.groupBox_potential->setEnabled(true);
}

bool comparePaird(paird elem1, paird elem2)
{
    return elem1.second < elem2.second;
}

void CSmileGUI::redrawFreqMap(bool show)
{
    if(!show || core->orbits==NULL) return;
    int nFMpoints = core->orbits->OrbitList.size();
    int numshell = ui.selectEnergyLevel_fm->value();
    if(numshell>0 && core->model!=NULL)
        ui.lineEdit_E->setText(QString::number(core->model->shellEnergy[numshell]));
    bool useConfigSpace = ui.checkBox_fmConfigSpace->isChecked() && (core->model!=0);
    if(ui.radioButton_fmmapdiff->isChecked() || ui.radioButton_fmmaplyapunov->isChecked())
    {
        vectord w1w3r(nFMpoints),  w2w3r(nFMpoints),  w1w3c(nFMpoints),  w2w3c(nFMpoints);
        int nr=0, nc=0;
        for(int i=0; i<nFMpoints; i++)
        {
            bool useOrbit=true;
            if(useConfigSpace)
            {
                if(core->orbits->OrbitList[i].state==2 
                    && core->orbits->OrbitList[i].orbitWeight>SCHW_MIN_ORBIT_WEIGHT_USED/core->orbits->OrbitList.size() 
                    && core->orbits->OrbitList[i].celltimes.size()==core->model->grid->cellsInModel+1)
                {
                    // find out whether this orbit contributes to density of i-th shell (or to whole model)
                    int cellmin=0, cellmax=core->model->grid->cellsInModel;
                    if(numshell>0)
                    {
                        cellmin = (numshell-1)*core->model->grid->cellsPerShell;
                        cellmax = numshell*core->model->grid->cellsPerShell;
                    }
                    double massinshell=0, massorbitinshell=0;
                    for(int c=cellmin; c<cellmax; c++)
                    {
                        massorbitinshell += core->orbits->OrbitList[i].celltimes[c]*core->orbits->OrbitList[i].orbitWeight;
                        massinshell += core->model->cellMass[c];
                    }
                    useOrbit = massorbitinshell/massinshell > SCHW_MIN_ORBIT_WEIGHT_DISPLAYED;
                }
                else
                    useOrbit=false;
            }
            else
            {
                useOrbit = (numshell==0) || core->orbits->OrbitList[i].shellIndex+1==numshell;  // use orbits which belong to given energy shell
            }
            if(core->orbits->OrbitList[i].state==2 && useOrbit)
            {
                bool criterion;
                if(ui.radioButton_fmmapdiff->isChecked()) 
                    criterion = core->orbits->OrbitList[i].lfccdiff <= config.chaoticMinFreqDiff;
                else 
                    criterion = core->orbits->OrbitList[i].lambda <= config.chaoticMinLambda;
                double denom = (core->N_dim==3)?core->orbits->OrbitList[i].lf[2]:1;
                if(denom==0) continue;
                if(criterion)
                {
                    w1w3r[nr] = core->orbits->OrbitList[i].lf[0]/denom;
                    w2w3r[nr] = core->orbits->OrbitList[i].lf[1]/denom;
                    nr++;
                }
                else
                {
                    w1w3c[nc] = core->orbits->OrbitList[i].lf[0]/denom;
                    w2w3c[nc] = core->orbits->OrbitList[i].lf[1]/denom;
                    nc++;
                }
            }
        }
        CurveFreqMapR->setData(&(w1w3r.front()), &(w2w3r.front()), nr);
        CurveFreqMapC->setData(&(w1w3c.front()), &(w2w3c.front()), nc);
        if(core->N_dim==3)
        {
            PlotFreqMap->setAxisScale(QwtPlot::yLeft, 0.5, 1.1);
            PlotFreqMap->setAxisScale(QwtPlot::xBottom, 0.3, 1.1);
        }
        PlotFreqMap->replot();
        PlotFreqMapHisto->hide();
        PlotFreqMapSS->hide();
        PlotFreqMap->show();

        if(!FreqMapRunning)
        {
          if(useConfigSpace)
          {
            int cellmin = numshell>0 ? (numshell-1)*core->model->grid->cellsPerShell : 0;
            int cellmax = numshell>0 ? numshell*core->model->grid->cellsPerShell-1 : core->model->grid->cellsInModel-1;
            ui.label_info->setPlainText(getOrbitPopulation(0, cellmin, cellmax));
          }
          else 
            ui.label_info->setPlainText(getOrbitPopulation(numshell));
        }
    }
    else if(ui.radioButton_fmhistodiff->isChecked() || ui.radioButton_fmhistolyapunov->isChecked())
    {
        std::vector<paird> values;
        double totalWeight=0;
        for(int i=0; i<nFMpoints; i++)
            if(core->orbits->OrbitList[i].state==2 && (numshell==0 || core->orbits->OrbitList[i].shellIndex+1==numshell))
            {
                double weight=(useConfigSpace ? core->orbits->OrbitList[i].orbitWeight : 0);
                totalWeight+=weight;
                values.push_back(paird(weight, ui.radioButton_fmhistodiff->isChecked() ? core->orbits->OrbitList[i].lfccdiff : core->orbits->OrbitList[i].lambda));
            }
        std::sort(values.begin(), values.end(), comparePaird);
        int npoints=values.size();
        if(npoints>0)
        {
            vectord val(npoints), np(npoints);
            double minval=1, tw=0;
            for(int i=0; i<npoints; i++)
            {
                val[i]=values[i].second;
                if(totalWeight)
                {
                    np[i]=tw/totalWeight;
                    tw+=values[i].first;
                }
                else
                    np[i]=i*1.0/npoints;
                if((val[i]>0) && (val[i]<minval)) minval=val[i];
            }
            CurveFreqMapHisto->setData(&(val.front()), &(np.front()), npoints);
            PlotFreqMapHisto->setAxisScale(QwtPlot::xBottom, minval/1.5, val[npoints-1]*1.1);
            PlotFreqMapHisto->replot();
        }
        PlotFreqMap->hide();
        PlotFreqMapSS->hide();
        PlotFreqMapHisto->show();
    }
    else if(ui.radioButton_fmssdiff->isChecked() || ui.radioButton_fmsslyapunov->isChecked() ||
        ui.radioButton_fmspdiff->isChecked() || ui.radioButton_fmsplyapunov->isChecked() ||
        ui.radioButton_fmsydiff->isChecked() || ui.radioButton_fmsylyapunov->isChecked())
    {
        vectord Xr(nFMpoints), Yr(nFMpoints), Xc(nFMpoints), Yc(nFMpoints);
        int nr=0, nc=0;
        double rad=0;
        for(int i=0; i<nFMpoints; i++)
        {
            if(core->orbits->OrbitList[i].state==2 && (numshell==0 || core->orbits->OrbitList[i].shellIndex+1==numshell))
            {
                bool criterion;
                if(ui.radioButton_fmssdiff->isChecked() || ui.radioButton_fmspdiff->isChecked() || ui.radioButton_fmsydiff->isChecked()) 
                    criterion = core->orbits->OrbitList[i].lfccdiff <= config.chaoticMinFreqDiff;
                else
                    criterion = core->orbits->OrbitList[i].lambda <= config.chaoticMinLambda;
                double X=core->orbits->OrbitList[i].ip[0];
                double Y=core->orbits->OrbitList[i].ip[1];
                double Z=core->orbits->OrbitList[i].ip[2];
                double Xp, Yp;
                if(ui.radioButton_fmsydiff->isChecked() || ui.radioButton_fmsylyapunov->isChecked())
                {
                    Xp=Y;
                    Yp=atan2(core->orbits->OrbitList[i].iv[0], core->orbits->OrbitList[i].iv[2]);
                }
                else
                    XYZtoXY(X,Y,Z,Xp,Yp);
                if(sqr(Xp)+sqr(Yp)>sqr(rad)) rad=sqrt(sqr(Xp)+sqr(Yp));
                if(ui.radioButton_fmssdiff->isChecked() || ui.radioButton_fmsslyapunov->isChecked())
                {
                    if(sqr(core->orbits->OrbitList[i].iv[0])+sqr(core->orbits->OrbitList[i].iv[1])+sqr(core->orbits->OrbitList[i].iv[2])>0) continue;   // this is not a stationary orbit
                }
                else if(ui.radioButton_fmspdiff->isChecked() || ui.radioButton_fmsplyapunov->isChecked())
                {
                    if(X*Y*Z!=0 || X==0 && Z==0) continue;   // this is not a principal-plane orbit
                }
                else
                {
                    if(X!=0 || Z!=0) continue;  // this point is not on Y axis
                }
                if(criterion)
                {
                    Xr[nr] = Xp;
                    Yr[nr] = Yp;
                    nr++;
                }
                else
                {
                    Xc[nc] = Xp;
                    Yc[nc] = Yp;
                    nc++;
                }
            }
        }
        CurveFreqMapSSR->setData(&(Xr.front()), &(Yr.front()), nr);
        CurveFreqMapSSC->setData(&(Xc.front()), &(Yc.front()), nc);
        PlotFreqMapSS->setAxisScale(QwtPlot::yLeft, -rad, rad);
        PlotFreqMapSS->setAxisScale(QwtPlot::xBottom, -rad, rad);
        PlotFreqMapSS->replot();
        PlotFreqMap->hide();
        PlotFreqMapHisto->hide();
        PlotFreqMapSS->show();
    }
}

void CSmileGUI::on_selectEnergyLevel_fm_valueChanged(int numShell)
{
    redrawFreqMap(true);
    if(numShell>0 && core->model!=NULL)
        ui.lineEdit_E->setText(QString::number(core->model->shellEnergy[numShell]));
}

void CSmileGUI::on_checkBox_fmConfigSpace_stateChanged(int)
{
    redrawFreqMap(true);
}

QString CSmileGUI::getOrbitPopulation(int numshellOrbit, int numcellMin, int numcellMax)
{
    if(core->orbits==NULL) return "Error!";
    else return core->orbits->getOrbitPopulation(numshellOrbit, numcellMin, numcellMax).replace("\t", "\n");
}

void CSmileGUI::drawFreqMapResonances()
{
    const int NR=18;   // number of resonances depicted on the map
    double res[NR][5] = {
        {0,1,-1 ,0.80,1},
        {1,-1,0 ,0.55,0.55},
        {2,-1,0 ,0.45,0.90},
        {3,-2,0 ,0.34,0.51},
        {4,-3,0 ,0.39,0.52},
        {2,0,-1 ,0.50,1.08},
        {3,0,-2 ,0.66,0.52},
        {1,-3,2 ,0.34,0.78},
        {1,-2,1 ,0.80,0.90},
        {2,-3,1 ,0.33,0.55},
        {4,-2,-1,0.78,1.08},
        {3,-1,-1,0.69,1.08},
        {3,1,-3 ,0.63,1.08},
        {2,1,-2 ,0.74,0.52},
        {2,2,-3 ,0.80,0.70},
        {1,2,-2 ,0.74,0.63},
        {1,1,-1 ,0.47,0.53} };
    double xmin=0.3, xmax=1., ymin=0.4, ymax=1.;
    double boundrect[4][3] = { {1, 0, -xmin}, {1, 0, -xmax}, {0, 1, -ymin}, {0, 1, -ymax} };
    //QString S;
    for(int i=0; i<NR; i++)
    {
        double x[2], y[2];
        if(res[i][1]==0)
        {
            x[0]=x[1]=-res[i][2]/res[i][0];
            y[0]=ymin;
            y[1]=ymax;
        }
        else if(res[i][0]==0)
        {
            x[0]=xmin;
            x[1]=xmax;
            y[0]=y[1]=-res[i][2]/res[i][1];
        }
        else
        {   // intersect with bounding rectangle
            std::vector<double> ax(0), ay(0);
            for(int c=0; c<4; c++)
            {
                double D = res[i][0]*boundrect[c][1] - res[i][1]*boundrect[c][0];
                ax.push_back( (-res[i][2]*boundrect[c][1] + res[i][1]*boundrect[c][2]) / D);
                ay.push_back(-(-res[i][2]*boundrect[c][0] + res[i][0]*boundrect[c][2]) / D);
            }
            std::sort(ax.begin(), ax.end());
            std::sort(ay.begin(), ay.end());
            int sgn = (res[i][0]*res[i][1]>0)?1:0;
            x[0]=ax[1]; x[1]=ax[2];
            y[0]=ay[1+sgn]; y[1]=ay[2-sgn];
        }
        QwtPlotCurve* c = new QwtPlotCurve();
        c->setPen(QPen(Qt::darkGray, 1, Qt::DotLine));
        c->setData(x, y, 2);
        c->attach(PlotFreqMap);
        QwtPlotMarker* m = new QwtPlotMarker();
        m->setValue(res[i][3], res[i][4]);
        m->setLabel(QString::number(res[i][0])+","+QString::number(res[i][1])+","+QString::number(res[i][2]));
        m->attach(PlotFreqMap);
    }
}

void CSmileGUI::FMclicked(const QPointF &pos)
{
    if(core->orbits==NULL || core->orbits->OrbitList.size()==0)
        return;
    int mode=-1;
    if(ui.radioButton_fmmapdiff->isChecked() || ui.radioButton_fmmaplyapunov->isChecked()) mode=0;
    else if(ui.radioButton_fmssdiff->isChecked() || ui.radioButton_fmsslyapunov->isChecked()) mode=1;
    else if(ui.radioButton_fmspdiff->isChecked() || ui.radioButton_fmsplyapunov->isChecked()) mode=2;
    if(mode<0) return;  // sould not happen however..
    double px=pos.x();
    double py=pos.y();
    double mindist=1; 
    int np=0;
    int numshell = ui.selectEnergyLevel_fm->value();
    for(int i=0; i<core->orbits->OrbitList.size(); i++)
    {
        double dist=1;
        if(mode==0)
        {
            double lf2 = (core->potential->N_dim==3)?core->orbits->OrbitList[i].lf[2]:1;
            double w1w3o = core->orbits->OrbitList[i].lf[0]/lf2;
            double w2w3o = core->orbits->OrbitList[i].lf[1]/lf2;
            dist = sqr(w1w3o-px) + sqr(w2w3o-py);
        }
        else
        {
            double X=core->orbits->OrbitList[i].ip[0];
            double Y=core->orbits->OrbitList[i].ip[1];
            double Z=core->orbits->OrbitList[i].ip[2];
            double Xp, Yp;
            XYZtoXY(X,Y,Z,Xp,Yp);
            if(mode==1 && X*Y*Z==0 || mode==2 && X*Y*Z!=0) continue; 
            dist = sqr(Xp-px) + sqr(Yp-py);
        }
        if(dist<mindist && (numshell==0 || core->orbits->OrbitList[i].shellIndex+1==numshell))
        {
            np=i;
            mindist=dist;
        }
    }
    if(mindist>=1) return;  // too far
    // load data from np'th orbit
    InternalRecalc=true;
    ui.lineEdit_x->setText(QString::number(core->orbits->OrbitList[np].ip[0], 'f', 7));
    ui.lineEdit_y->setText(QString::number(core->orbits->OrbitList[np].ip[1], 'f', 7));
    ui.lineEdit_z->setText(QString::number(core->orbits->OrbitList[np].ip[2], 'f', 7));
    ui.lineEdit_vx->setText(QString::number(core->orbits->OrbitList[np].iv[0], 'f', 7));
    ui.lineEdit_vy->setText(QString::number(core->orbits->OrbitList[np].iv[1], 'f', 7));
    ui.lineEdit_vz->setText(QString::number(core->orbits->OrbitList[np].iv[2], 'f', 7));
    InternalRecalc=false;
    bool tmpUseE=core->useICe;
    core->useICe=false;  // need to put in values for IC, not for E
    ICchanged();
    core->useICe=tmpUseE;
    // set exact values
    core->icX=core->orbits->OrbitList[np].ip[0];
    core->icY=core->orbits->OrbitList[np].ip[1];
    core->icZ=core->orbits->OrbitList[np].ip[2];
    core->icVx=core->orbits->OrbitList[np].iv[0];
    core->icVy=core->orbits->OrbitList[np].iv[1];
    core->icVz=core->orbits->OrbitList[np].iv[2];
    ui.label_info->setPlainText("lfX="+QString::number(core->orbits->OrbitList[np].lf[0], 'f', 7) + "\nlfY="+QString::number(core->orbits->OrbitList[np].lf[1], 'f', 7) + 
        ((core->N_dim==3)? "\nlfZ="+QString::number(core->orbits->OrbitList[np].lf[2], 'f', 7) : "") +
        "\n" + core->orbits->OrbitList[np].Description + 
        "\nFreqDiff=" + QString::number(core->orbits->OrbitList[np].lfccdiff, 'g', 3) +
        (core->orbits->OrbitList[np].calcLyapunov? "\nLyapunovExp=" + QString::number(core->orbits->OrbitList[np].lambda, 'g', 3) : "") +
        (core->model? "\nOrbit weight=" + QString::number(core->orbits->OrbitList[np].orbitWeight, 'g', 3) + (core->orbits->OrbitList[np].isChaotic?"; chaotic":"; regular") : "") +
        "\nInt.time="+QString::number(core->orbits->OrbitList[np].time/core->orbits->OrbitList[np].timeUnit, 'f', 0)+" Torb");
}

void CSmileGUI::XYZtoXY(const double X, const double Y, const double Z, double& x, double& y)
{
    x = (Y-X)/sqrt(2.0);
    y = (2*Z-X-Y)/sqrt(6.0);
}


/// ----------------------------  Schwarzschild modelling --------- ///
void CSmileGUI::on_ButtonSchwModel_clicked()
{
    if(FreqMapRunning || ExportNbodyRunning) return;
    if(core->SchwThr!=NULL)  // stop
    {
        emit stopSchw();
        return;
    }
    if(SchwModelRunning)
    {
        emit KillFreqMapThreads();
        return;
    }
    if(core->N_dim!=3)
        return;

    // start building library
    core->BuildSchwIC(ui.checkBox_smExistingSS->isChecked());
    core->StartOrbitLibrary(SLOT(coreSchwOrbitLibraryFinished()));
    ui.groupBox_N_dim->setEnabled(false);
    ui.groupBox_potential->setEnabled(false);
    ui.groupBox_smOptim->setEnabled(false);
    ui.groupbox_smGrid->setEnabled(false);
    ui.ButtonSchwModel->setText("Stop");
    timeFM.start();
    ui.progressBarSM->setRange(0, core->orbits->OrbitList.size());
    ui.progressBarSM->setValue(0);
    ui.progressBarSM->show();
    SchwModelRunning=true;
    for(int t=0; t<core->CalcManyThr.size(); t++)
        connect(this, SIGNAL(KillFreqMapThreads()), core->CalcManyThr[t], SLOT(stopThread()));
}

void CSmileGUI::SchwOrbitLibraryFinished()    // slot called when core->signalSchwOrbitLibraryFinished() emitted
{
    SchwModelRunning=false;
    ui.ButtonSchwModel->setText("Start");
    ui.progressBarSM->hide();
    ui.groupBox_N_dim->setEnabled(true);
    ui.groupBox_potential->setEnabled(true);
    ui.groupBox_smOptim->setEnabled(true);
    ui.groupBox_smOptimParams->setEnabled(true);
    ui.groupbox_smGrid->setEnabled(true);
    ui.ButtonSchwLP->setEnabled(true);
    ui.ButtonSchwQP->setEnabled(true);
    ui.ButtonSchwLucy->setEnabled(true);
    ui.ButtonSchwExportGrid->setEnabled(true);
    ui.ButtonSchwNbody->setEnabled(true);
    redrawFreqMap(true);
    ui.label_info->setPlainText(QString::number(core->orbits->nComplete)+" orbits\n" +
        QString::number(core->orbits->nComplete*1000.0/timeFM.elapsed(),'g',2)+" orbits/second");
    ui.label_info->setToolTip("");
    ui.label_info->repaint();
}

void CSmileGUI::SchwExportNbodyFinished()    // slot called when core->signalSchwExportNbodyFinished() emitted
{
    ExportNbodyRunning=false;
    ui.tabWidget_sm->setEnabled(true);
    ui.ButtonSchwNbody->setText("Create Nbody IC");
    ui.progressBarSM->hide();
}

void CSmileGUI::on_ButtonSchwNbody_clicked()
{
    if(SchwModelRunning || FreqMapRunning || core->orbits==NULL || core->orbits->OrbitList.size()==0 || core->model==NULL)
        return;
    if(ExportNbodyRunning)
    {   // stop export (determine which stage we are at now)
        if(core->CalcManyThr.size()>0)
        {
            ui.label_info->setPlainText("Sorry, cannot stop reintegration..:(");
        }
        if(core->SchwThr!=NULL)  // stop
        {
            emit stopSchw();
        }
        return;
    }
    bool ok;
    int NPoints=QInputDialog::getInteger(this, "Export to N-body", "Enter number of points", 1000000, 0, 1e8, 1, &ok);
    if(!ok) return;
    QString fileName = QFileDialog::getSaveFileName(this, "Export to N-body", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);
    int refineFactor=QInputDialog::getInteger(this, "Export to N-body", "Mass refine factor (2^N)", 0, 0, 6, 1, &ok);
    if(!ok) refineFactor=0;

    connect(this, SIGNAL(stopSchw()), core->model, SLOT(stop()));
    ExportNbodyRunning=true;
    ui.tabWidget_sm->setEnabled(false);
    ui.ButtonSchwNbody->setText("Cancel");
    ui.progressBarSM->setValue(0);
    ui.progressBarSM->setRange(0, core->orbits->OrbitList.size());
    ui.progressBarSM->show();
    core->SchwExportNbody(NPoints, fileName, refineFactor);
}

void CSmileGUI::on_ButtonSchwLP_clicked()
{
    if(core->SchwThr!=NULL || core->model==NULL) return;
    ui.ButtonSchwLP->setEnabled(false);
    ui.ButtonSchwQP->setEnabled(false);
    ui.ButtonSchwLucy->setEnabled(false);
    ui.groupBox_smOptimParams->setEnabled(false);
    core->SchwStartOptimization(CSchwarzschildThread::SM_LINEAR);
}

void CSmileGUI::on_ButtonSchwQP_clicked()
{
    if(core->SchwThr!=NULL || core->model==NULL) return;
    ui.ButtonSchwLP->setEnabled(false);
    ui.ButtonSchwQP->setEnabled(false);
    ui.ButtonSchwLucy->setEnabled(false);
    ui.groupBox_smOptimParams->setEnabled(false);
    core->SchwStartOptimization(CSchwarzschildThread::SM_QUADRATIC);
}

void CSmileGUI::on_ButtonSchwLucy_clicked()
{
    if(core->model==NULL) return;
    if(core->SchwThr!=NULL)  // stop
        emit stopSchw();
    else
    {
        connect(this, SIGNAL(stopSchw()), core->model, SLOT(stop()));
        ui.radioButton_smviewweights->setChecked(true);
        ui.ButtonSchwLP->setEnabled(false);
        ui.ButtonSchwQP->setEnabled(false);
        ui.ButtonSchwLucy->setText("Stop");
        ui.groupBox_smOptimParams->setEnabled(false);
        core->SchwStartOptimization(CSchwarzschildThread::SM_LUCY);
    }
}

void CSmileGUI::SchwOptimizationFinished(const QString& message)
{
    ui.ButtonSchwLP->setEnabled(true);
    ui.ButtonSchwQP->setEnabled(config.sm_useBPMPD);
    ui.ButtonSchwLucy->setEnabled(true);
    ui.ButtonSchwLucy->setText("Lucy");
    ui.groupBox_smOptimParams->setEnabled(true);
    ui.groupBox_smOptim->setEnabled(true);
    SchwModelResult = message;
    redrawSchwModel(true);
}

void CSmileGUI::on_selectEnergyLevel_valueChanged(int value)
{
    redrawSchwModel(true);
}

void CSmileGUI::redrawSchwModel(bool updateInfo)
{
    if(!core->model)
        return;
    int numShell=ui.selectEnergyLevel->value();
    if(numShell>0)
        ui.lineEdit_E->setText(QString::number(core->model->shellEnergy[numShell]));
    if(ui.radioButton_smviewgrid->isChecked())
    {
        int npoints=(numShell==0 ? core->model->grid->cellsInModel : core->model->grid->cellsPerShell);
        int nstart =(numShell==0 ? 0 : (numShell-1)*core->model->grid->cellsPerShell);
        int nend   =(numShell==0 ? core->model->grid->cellsInModel : numShell*core->model->grid->cellsPerShell);
        vectord CellsFx(npoints), CellsFy(npoints), CellsIx(npoints), CellsIy(npoints);
        int nf=0, ni=0;
        double rad=0;
        for(int c=nstart; c<nend; c++)
        {
            double X, Y, Z, Xp, Yp;
            core->model->grid->cellCenter(c, X, Y, Z);
            XYZtoXY(X,Y,Z,Xp,Yp);
            if(sqr(Xp)+sqr(Yp)>sqr(rad)) rad=sqrt(sqr(Xp)+sqr(Yp));
            if(core->model->cellPenalty[c]!=0)
            {
                CellsIx[ni]=Xp;
                CellsIy[ni]=Yp;
                ni++;
            }
            else
            {
                CellsFx[nf]=Xp;
                CellsFy[nf]=Yp;
                nf++;
            }
        }
        CurveSchwCellF->setData(&(CellsFx.front()), &(CellsFy.front()), nf);
        CurveSchwCellI->setData(&(CellsIx.front()), &(CellsIy.front()), ni);
        PlotSchw->setAxisScale(QwtPlot::yLeft, -rad, rad);
        PlotSchw->setAxisScale(QwtPlot::xBottom, -rad, rad);
        PlotSchw->replot();
        PlotSchwHisto->hide();
        PlotSchw->show();
        if(updateInfo)  ui.label_info->setPlainText(SchwModelResult + 
            getOrbitPopulation(0, nstart, nend-1));
    }
    else if(ui.radioButton_smviewbeta->isChecked())
    {   // plot anisotropy coefficient beta
        int npoints=(numShell==0 ? core->model->grid->cellsInModel : core->model->grid->cellsPerShell);
        int nstart =(numShell==0 ? 0 : (numShell-1)*core->model->grid->cellsPerShell);
        int nend   =(numShell==0 ? core->model->grid->cellsInModel : numShell*core->model->grid->cellsPerShell);
        vectord val(npoints), np(npoints);
        for(int c=nstart; c<nend; c++)
        {
            int ishell=c/core->model->grid->cellsPerShell;
            double vr2=0, vt2=0;
            for(int o=0; o<core->orbits->OrbitList.size(); o++)
                if(core->orbits->OrbitList[o].state==2 && (numShell==0 || core->orbits->OrbitList[o].shellIndex+1==numShell) && core->orbits->OrbitList[o].celltimes[c]>0 && core->orbits->OrbitList[o].orbitWeight>0)
                {
                    vr2 += core->orbits->OrbitList[o].cellVr[c] * core->orbits->OrbitList[o].celltimes[c] * core->orbits->OrbitList[o].orbitWeight;
                    vt2 += core->orbits->OrbitList[o].cellVt[c] * core->orbits->OrbitList[o].celltimes[c] * core->orbits->OrbitList[o].orbitWeight;
                }
            if(vr2>0)
                val[c] = 1-vt2/vr2/2;
            else
                val[c] = 1;
            np[c]=c*1.0;
        }
        CurveSchwHistoR->setData(&(np.front()), &(val.front()), npoints);
        CurveSchwHistoC->setData(NULL, NULL, 0);
        PlotSchwHisto->setAxisScale(QwtPlot::yLeft, -1, 1);
        PlotSchwHisto->setAxisAutoScale(QwtPlot::xBottom);
        CurveSchwHistoR->setStyle(QwtPlotCurve::Dots);
        PlotSchwHisto->setAxisScaleEngine(QwtPlot::yLeft, new QwtLinearScaleEngine);
        PlotSchwHisto->replot();
        PlotSchw->hide();
        PlotSchwHisto->show();
        if(updateInfo)  ui.label_info->setPlainText(SchwModelResult + 
            getOrbitPopulation(numShell));
    }
    else if(ui.radioButton_smviewweights->isChecked() || ui.radioButton_smviewhisto->isChecked())
    {   // plot histogram of orbit weights
        vectord valuesR, valuesC, npR, npC;
        double minval=1, maxval=0;
        for(int i=0; i<core->orbits->OrbitList.size(); i++)
            if(core->orbits->OrbitList[i].state==2 && (numShell==0 || core->orbits->OrbitList[i].shellIndex+1==numShell))
            {
                if(core->orbits->OrbitList[i].isChaotic && !ui.radioButton_smviewhisto->isChecked())
                {
                    valuesC.push_back(core->orbits->OrbitList[i].orbitWeight);
                    npC.push_back(i*1.0);
                }
                else
                {
                    valuesR.push_back(core->orbits->OrbitList[i].orbitWeight);
                    npR.push_back(i*1.0);
                }
                if((core->orbits->OrbitList[i].orbitWeight>0) && (core->orbits->OrbitList[i].orbitWeight<minval)) minval=core->orbits->OrbitList[i].orbitWeight;
                if((core->orbits->OrbitList[i].orbitWeight>0) && (core->orbits->OrbitList[i].orbitWeight>maxval)) maxval=core->orbits->OrbitList[i].orbitWeight;
            }
        if(ui.radioButton_smviewhisto->isChecked() || npC.size()==0) 
        {
            std::sort(valuesR.begin(), valuesR.end());
            CurveSchwHistoC->setData(NULL, NULL, 0);
        }
        else
            CurveSchwHistoC->setData(&(npC.front()), &(valuesC.front()), valuesC.size());
        CurveSchwHistoR->setData(&(npR.front()), &(valuesR.front()), valuesR.size());
        CurveSchwHistoR->setStyle(ui.radioButton_smviewhisto->isChecked() ? QwtPlotCurve::Steps : QwtPlotCurve::Dots);
        PlotSchwHisto->setAxisScale(QwtPlot::yLeft, minval, maxval);
        PlotSchwHisto->setAxisAutoScale(QwtPlot::xBottom);
        PlotSchwHisto->setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine);
        PlotSchwHisto->replot();
        PlotSchw->hide();
        PlotSchwHisto->show();
        if(updateInfo)  ui.label_info->setPlainText(SchwModelResult + 
            getOrbitPopulation(numShell));
    }
}

void CSmileGUI::SMclicked(const QPointF &pos)
{
    if(core->orbits==NULL || core->orbits->OrbitList.size()==0 || core->model==NULL || core->model->grid==NULL)
        return;
    double px=pos.x();
    double py=pos.y();
    double mindist=1; 
    int nc=-1;
    int index=ui.selectEnergyLevel->value()-1;
    int minc=(index<0)?0 : index*core->model->grid->cellsPerShell;
    int maxc=(index<0)?core->model->grid->cellsInModel : (index+1)*core->model->grid->cellsPerShell;
    for(int c=minc; c<maxc; c++)
    {
        double dist=1;
        double X, Y, Z, Xp, Yp;
        core->model->grid->cellCenter(c, X, Y, Z);
        XYZtoXY(X,Y,Z,Xp,Yp);
        dist = sqr(Xp-px) + sqr(Yp-py);
        if(dist<mindist)
        {
            nc=c;
            mindist=dist;
        }
    }
    if(nc<0) return;
    int numorbits=0, numused=0;
    double mass=0;
    for(int o=0; o<core->orbits->OrbitList.size(); o++)
        if(core->orbits->OrbitList[o].celltimes[nc]>0)
        {
            numorbits++;
            if(core->orbits->OrbitList[o].orbitWeight > SCHW_MIN_ORBIT_WEIGHT_USED/core->orbits->OrbitList.size() ) numused++;
            mass += core->orbits->OrbitList[o].orbitWeight * core->orbits->OrbitList[o].celltimes[nc];
        }
    ui.label_info->setPlainText(QString::number(numorbits) + " orbits visit cell,\n" +
        QString::number(numused) + " orbits used in model,\n" +
        "Mass difference: " + QString::number((mass/core->model->cellMass[nc]-1) * 100, 'f', 4) + "%\n" +
        getOrbitPopulation(0, nc, nc));
}

void CSmileGUI::SMHistoClicked(const QPointF &pos)
{
    if(core->orbits==NULL || core->orbits->OrbitList.size()==0 || core->model==NULL || core->model->grid==NULL || !ui.radioButton_smviewweights->isChecked())
        return;
    int numShell = ui.selectEnergyLevel_fm->value();
    double px=pos.x();
    double py=pos.y();
    // scale clicked value also to [0..1] interval
    double minyval=1, maxyval=0;
    int minxval=core->orbits->OrbitList.size(), maxxval=0;
    for(int i=0; i<core->orbits->OrbitList.size(); i++)
        if(core->orbits->OrbitList[i].state==2 && (numShell==0 || core->orbits->OrbitList[i].shellIndex+1==numShell))
        {
            if(i<minxval) minxval=i;
            if(i>maxxval) maxxval=i;
            if(core->orbits->OrbitList[i].orbitWeight>0)
            {
                if(core->orbits->OrbitList[i].orbitWeight<minyval) minyval=core->orbits->OrbitList[i].orbitWeight;
                if(core->orbits->OrbitList[i].orbitWeight>maxyval) maxyval=core->orbits->OrbitList[i].orbitWeight;
            }
        }
    if(maxxval==0 || maxyval==0) return;
    minyval=log10(minyval);
    maxyval=log10(maxyval);
    px = (px-minxval)*1.0/(maxxval-minxval);
    py = (log10(py)-minyval)/(maxyval-minyval);
    // find nearest point
    double mindist=1; 
    int np=0;
    for(int i=0; i<core->orbits->OrbitList.size(); i++)
    {
        double dist=1;
        if(core->orbits->OrbitList[i].orbitWeight>0)
        {
            double ox=(i-minxval)*1.0/(maxxval-minxval);
            double oy=(log10(core->orbits->OrbitList[i].orbitWeight)-minyval)/(maxyval-minyval);
            dist = sqr(ox-px) + sqr(oy-py);
        }
        if(dist<mindist && (numShell==0 || core->orbits->OrbitList[i].shellIndex+1==numShell))
        {
            np=i;
            mindist=dist;
        }
    }
    if(mindist>=1) return;  // too far
    // load data from np'th orbit
    InternalRecalc=true;
    ui.lineEdit_x->setText(QString::number(core->orbits->OrbitList[np].ip[0], 'f', 7));
    ui.lineEdit_y->setText(QString::number(core->orbits->OrbitList[np].ip[1], 'f', 7));
    ui.lineEdit_z->setText(QString::number(core->orbits->OrbitList[np].ip[2], 'f', 7));
    ui.lineEdit_vx->setText(QString::number(core->orbits->OrbitList[np].iv[0], 'f', 7));
    ui.lineEdit_vy->setText(QString::number(core->orbits->OrbitList[np].iv[1], 'f', 7));
    ui.lineEdit_vz->setText(QString::number(core->orbits->OrbitList[np].iv[2], 'f', 7));
    InternalRecalc=false;
    bool tmpUseE=core->useICe;
    core->useICe=false;  // need to put in values for IC, not for E
    ICchanged();
    core->useICe=tmpUseE;
    // set exact values
    core->icX=core->orbits->OrbitList[np].ip[0];
    core->icY=core->orbits->OrbitList[np].ip[1];
    core->icZ=core->orbits->OrbitList[np].ip[2];
    core->icVx=core->orbits->OrbitList[np].iv[0];
    core->icVy=core->orbits->OrbitList[np].iv[1];
    core->icVz=core->orbits->OrbitList[np].iv[2];
    ui.label_info->setPlainText("lfX="+QString::number(core->orbits->OrbitList[np].lf[0], 'f', 7) + "\nlfY="+QString::number(core->orbits->OrbitList[np].lf[1], 'f', 7) + 
        ((core->N_dim==3)? "\nlfZ="+QString::number(core->orbits->OrbitList[np].lf[2], 'f', 7) : "") +
        "\n" + core->orbits->OrbitList[np].Description + 
        "\nFreqDiff=" + QString::number(core->orbits->OrbitList[np].lfccdiff, 'g', 3) +
        (core->orbits->OrbitList[np].calcLyapunov? "\nLyapunovExp=" + QString::number(core->orbits->OrbitList[np].lambda, 'g', 3) : "") +
        (core->model? "\nOrbit weight=" + QString::number(core->orbits->OrbitList[np].orbitWeight, 'g', 3) + (core->orbits->OrbitList[np].isChaotic?"; chaotic":"; regular") : "") +
        "\nInt.time="+QString::number(core->orbits->OrbitList[np].time/core->orbits->OrbitList[np].timeUnit, 'f', 0)+" Torb");
}

///-------------  export-import -------------///
void CSmileGUI::on_ButtonExportFM_clicked()
{
    if(core->orbits==NULL || core->orbits->OrbitList.size()==0)
        return;
    QString fileName = QFileDialog::getSaveFileName(this, "Export Frequency map", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);

    if(!core->orbits->exportOrbits(fileName, false))
        QMessageBox::warning(this, "Warning", "Cannot write file "+fileName);
    core->saveSettings(fileName+".ini");
}

void CSmileGUI::on_ButtonImportFM_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Import Frequency map", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);
    if(core->orbits)
        delete core->orbits;
    loadApplySettings(fileName+".ini");   // try loading saved potential and other data
    core->orbits=new COrbitLibrary(core->potential, NULL);
    if (!core->orbits->importOrbits(fileName, false)) {
        QMessageBox::warning(this, "Warning", "Cannot read file "+fileName);
        return;
    }
    redrawFreqMap(true);
    ui.label_info->setPlainText(QString::number(core->orbits->OrbitList.size())+" orbits loaded");
}

void CSmileGUI::on_ButtonExportSM_clicked()
{
    if(core->orbits==NULL || core->orbits->OrbitList.size()==0 || core->model==NULL)
        return;
    QString fileName = QFileDialog::getSaveFileName(this, "Export Schwarzschild model", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);

    if(!core->orbits->exportOrbits(fileName, true))
        QMessageBox::warning(this, "Warning", "Cannot write file "+fileName);
    core->saveSettings(fileName+".ini");
}

void CSmileGUI::on_ButtonImportSM_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Import Schwarzschild model", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);
    if(core->orbits)
        delete core->orbits;
    if(core->model)
        delete core->model;
    loadApplySettings(fileName+".ini");   // try loading saved potential and other data
    core->model=new CSchwarzschildModel(core->potential, config.sm_numShells, config.sm_linesPerSegment);
    core->orbits=new COrbitLibrary(core->potential, core->model->grid);
    if (!core->orbits->importOrbits(fileName, true)) 
    {
        QMessageBox::warning(this, "Warning", "Cannot read file "+fileName);
        return;
    }
    core->model->orbits=core->orbits; 
    ui.groupBox_smOptim->setEnabled(true);
    ui.groupBox_smOptimParams->setEnabled(true);
    ui.ButtonSchwLP->setEnabled(true);
    ui.ButtonSchwQP->setEnabled(config.sm_useBPMPD);
    ui.ButtonSchwLucy->setEnabled(true);
    ui.ButtonSchwExportGrid->setEnabled(true);
    ui.ButtonSchwNbody->setEnabled(true);
    double diff=core->model->calcCellPenalty();
    redrawFreqMap(true);
    on_selectEnergyLevel_valueChanged(ui.selectEnergyLevel->value());  // redraw grid
    ui.label_info->setPlainText(QString::number(core->orbits->OrbitList.size())+" orbits loaded\n" + QString::number(diff) + " total difference");
}

void CSmileGUI::on_ButtonSchwExportGrid_clicked()
{
    if(core->model==NULL)
        return;
    QString fileName = QFileDialog::getSaveFileName(this, "Export Schwarzschild grid", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);

    if(!core->model->exportGrid(fileName))
        QMessageBox::warning(this, "Warning", "Cannot write file "+fileName);
}

void CSmileGUI::on_ButtonExportOrbit_clicked()
{
    if(!core->orbit || !core->orbit->tp[0].size())
        return;
    QString fileName = QFileDialog::getSaveFileName(this, "Export orbit", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);
    if(!core->exportOrbit(fileName))
        QMessageBox::warning(this, "Warning", "Cannot write file "+fileName);
}

void CSmileGUI::on_ButtonImportOrbit_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Import orbit", core->WorkDir);
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);
    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) 
        return;
    QTextStream strm(&file);

    vectord pv[2*N_DIM];
    QStringList fields;
    double maxr2=0;
    while(!strm.atEnd())
    {
        QString ln = strm.readLine();
        fields=ln.split(QRegExp("\\s+"));
        bool ok=false;
        if(fields.size()>=7 && (fields[0].toDouble(&ok),ok))
        {
            for(int i=0; i<2*N_DIM; i++)
            {
                pv[i].push_back(fields[i+1].toDouble());
            }
            double r2=sqr(pv[0].back())+sqr(pv[1].back())+sqr(pv[2].back());
            if(r2>maxr2) maxr2=r2;
        }
    }
    if(core->orbit!=NULL)
    {
        delete core->orbit->potential;  // manually delete, since it was created by cloning externally to the object
        delete core->orbit;
    }
    double IP[N_DIM], IV[N_DIM];
    IP[0]=pv[0][0];  IP[1]=pv[1][0];  IP[2]=pv[2][0];
    IV[0]=pv[3][0];  IV[1]=pv[4][0];  IV[2]=pv[5][0];
    core->icX=IP[0]; core->icY=IP[1]; core->icZ=IP[2];
    core->icVx=IV[0]; core->icVy=IV[1]; core->icVz=IV[2];
    bool useE=core->useICe;
    core->useICe=false;
    core->initIC();
    core->useICe=useE;
    double intTime =fields[0].toDouble();
    double timeUnit=intTime/(pv[1].size()-1);
    config.intTimeStepsPerPeriod = core->Torbx/timeUnit;
    config.intTimeInPeriods = (intTime+timeUnit)/core->Torbx;
    InternalRecalc=true;
    ui.lineEdit_x->setText(QString::number(core->icX));
    ui.lineEdit_y->setText(QString::number(core->icY));
    ui.lineEdit_z->setText(QString::number(core->icZ));
    ui.lineEdit_vx->setText(QString::number(core->icVx));
    ui.lineEdit_vy->setText(QString::number(core->icVy));
    ui.lineEdit_vz->setText(QString::number(core->icVz));
    ui.lineEdit_E->setText(QString::number(core->E));
    ui.lineEdit_intTime->setText(QString::number(config.intTimeInPeriods));
    ui.lineEdit_timeStep->setText(QString::number(config.intTimeStepsPerPeriod));
    ui.spinBox_intervalCount->setValue(1);
    InternalRecalc=false;
    core->orbit = new COrbit(core->potential->clone(), NULL, IP, IV, core->Torbx/config.intTimeStepsPerPeriod, core->Torbx, config.calcLyapunov, config.usePS);
    core->orbit->nActual=pv[0].size();
    core->orbit->tp[0]=pv[0];
    core->orbit->tp[1]=pv[1];
    core->orbit->tp[2]=pv[2];
    core->orbit->tv[0]=pv[3];
    core->orbit->tv[1]=pv[4];
    core->orbit->tv[2]=pv[5];
    core->orbit->maxr=sqrt(maxr2);
    core->orbit->intTime=config.intTimeInPeriods*core->Torbx;
    refreshOrbitAndSpectrum();
}

void CSmileGUI::on_ButtonPrint_clicked()
{
    /*for(int c=1; c<=ui.spinBox_intervalCount->value(); c++)
    {
        ui.spinBox_IntervalNum->setValue(c);
        QImage P(364,355,QImage::Format_RGB888);
        QwtPlotPrintFilter F=QwtPlotPrintFilter();
        PlotOrbit->print(P, F);
        P.save(QString::number(c+1000)+".png");
    }
    return;*/
    QwtPlot* fig = NULL;  
    if(PlotOrbit->isVisible())
        fig = PlotOrbit;
    if(PlotPoincare->isVisible())
        fig = PlotPoincare;
    if(PlotFreq->isVisible())
        fig = PlotFreq;
    if(PlotLyapunov->isVisible())
        fig = PlotLyapunov;
    if(PlotFreqMap->isVisible())
        fig = PlotFreqMap;
    if(PlotFreqMapHisto->isVisible())
        fig = PlotFreqMapHisto;
    if(PlotFreqMapSS->isVisible())
        fig = PlotFreqMapSS;
    if(PlotSchw->isVisible())
        fig = PlotSchw;
#ifdef USEQWT3D
    bool Plot3dVisible=Plot3d->isVisible();
    // 3d plot is not inherited from QwtPlot, so treat it differently
#else
    bool Plot3dVisible=false;
#endif
    if(fig==NULL && !Plot3dVisible)
        return;
    QString fileName = QFileDialog::getSaveFileName(this, "Print figure to file", core->WorkDir, "PostScript (*.ps);;PDF (*.pdf)");
    if (fileName.isEmpty())
        return;
    core->WorkDir = core->getDirName(fileName);
    if(fig)
    {
        QPrinter P;
        P.setOutputFileName(fileName);
        fig->print(P);
    }
#ifdef USEQWT3D
    else if(Plot3dVisible)
    {
        QString Format=fileName.endsWith(".pdf", Qt::CaseInsensitive) ? "PDF" : "PS";
        VectorWriter* printer = (VectorWriter*)IO::outputHandler(Format);
        printer->setTextMode(VectorWriter::NATIVE);
        printer->setSortMode(VectorWriter::BSPSORT);
        printer->setLandscape(VectorWriter::OFF);
        IO::save(Plot3d, fileName, Format);
    }
#endif
}

///------------- Triangulation thread helper class --------------///
#ifdef USE3DFACETS
CTriangThread::CTriangThread()
{
    finish = restart = false;
}

CTriangThread::~CTriangThread()
{
    mutex.lock();
    finish=restart=true;
    condition.wakeOne();
    mutex.unlock();
}
 
void CTriangThread::run()
{
    while(!finish)
    {
        createTriangulation();
        mutex.lock();
        if(!restart)
            condition.wait(&mutex);
        restart=false;
        mutex.unlock();
    }
}

void CTriangThread::startTriangulation(const std::vector<double> *_trajp, double _maxdist, int _slice)
{
    QMutexLocker locker(&mutex);
    // store data 
    nPointsMy = _trajp[0].size();
    for(int d=0; d<N_DIM; d++)
        trajpMy[d] = _trajp[d];
    sliceMy = _slice;
    maxdistMy = _maxdist;
    // start or restart thread
    if(!isRunning())
        start(QThread::LowPriority);
    else
    {    
        restart=true;
        condition.wakeOne();
    }
}

bool CTriangThread::toolong(int v1, int v2)
{
    return (sqr(trjpos[v1].x-trjpos[v2].x) + sqr(trjpos[v1].y-trjpos[v2].y) + sqr(trjpos[v1].z-trjpos[v2].z) > maxdist);
}
void CTriangThread::addface(int v1, int v2, int v3)
{
    Triangle T(v1, v2, v3);
    if(Triangles.remove(T)) 
        T.dup=true;
    Triangles << T;
}

void CTriangThread::createTriangulation()
{
    {
        Triangles.clear();
        // before proceeding further, copy data to local variables
        mutex.lock();
        facets.clear();
        trjpos.clear();
        maxdist=maxdistMy;
        int coord = sliceMy-1;
        int nPoints=0;
        for(int i=0; i<nPointsMy; i++)
        {
            if((sliceMy==0) || (trajpMy[coord][i]>=0))
            {
                trjpos.push_back(Qwt3D::Triple(trajpMy[0][i], trajpMy[1][i], trajpMy[2][i]));
                nPoints++;
            }
        }
        mutex.unlock();

        QProcess process;
        process.setReadChannel(QProcess::StandardOutput);
        process.start(core->AppDir+"qdelaunay", QStringList("i"));
        if(!process.waitForStarted()) return;
        QTextStream strmI(&process);
        strmI << "3" << endl << nPoints << endl;
        for(int i=0; i<nPoints; i++)
        {
            strmI << trjpos[i].x << " " << trjpos[i].y << " " << trjpos[i].z << endl;
            if(restart) return;
        }
        process.closeWriteChannel();
        int nttot=0, nt=0;
        if(process.waitForFinished())
        {
            QTextStream strmO(&process);
            QString ln = strmO.readLine();
            nttot = ln.toInt();
            while(!strmO.atEnd())
            {
                ln = strmO.readLine();
                QStringList abcd=ln.split(" ");
                if(abcd.size() >= 4)
                {
                    int v1=abcd[0].toInt();
                    int v2=abcd[1].toInt();
                    int v3=abcd[2].toInt();
                    int v4=abcd[3].toInt();
                    if((v1>=nPoints) || (v2>=nPoints) || (v3>=nPoints) || (v4>=nPoints)) continue;
                    if(!(toolong(v1,v2) || toolong(v1,v3) || toolong(v1,v4) ||
                         toolong(v2,v3) || toolong(v2,v4) || toolong(v3,v4)))
                    {
                        addface(v1, v2, v3);
                        addface(v1, v2, v4);
                        addface(v1, v3, v4);
                        addface(v2, v3, v4);
                        nt++;
                    }
                }
                if(restart) return;
            }
        }
        // update faces
        int ntot=0, nout=0;
        QList<Triangle> TList = Triangles.toList();
        Qwt3D::Cell tricell(3);
        foreach (Triangle T, TList)
        {
            ntot++;
            if(!T.dup)
            {
                tricell[0] = T.c1;
                tricell[1] = T.c2;
                tricell[2] = T.c3;
              //  if((MyOrbit->tp[0][T.c1]>0) && (MyOrbit->tp[0][T.c2]>0) && (MyOrbit->tp[0][T.c3]>0))
                    facets.push_back(tricell);
                nout++;
            }
            if(restart) return;
        }
        emit finished(true);
    }
}

#endif

}
