// This file holds expressions for potential, forces and density for various models.
#pragma once
#include "defines.h"

namespace smile{

class CPotential
{
public:
enum POTENTIALTYPE
{
    PT_LOG,
    PT_HARMONIC,
    PT_DEHNEN,
    PT_DEHNENBSE,
    PT_SCALEFREE,
    PT_SCALEFREEBSE,
    PT_BSE,
    PT_PLUMMERBSE,
    PT_NB
};
    const int N_dim;      // dimensions for current task (2 or 3)
    double q, p;          // axis ratio (y/x and z/x)  for potential (Log) or density (Dehnen model)
    const double Mbh;     // black hole mass

    CPotential();   // overriden; 
    CPotential(int _N_dim, double _q, double _p, double _Mbh);
    virtual ~CPotential();
    virtual CPotential* clone() const=0;
    virtual POTENTIALTYPE PotentialType() const=0;
    virtual char* PotentialName() const=0;
    virtual double Rho(double X, double Y, double Z, double t=0) const=0;
    virtual double Phi(double X, double Y, double Z, double t=0) const=0;
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const=0;
    double longaxisradius(double E) const;  // find maximal elongation of long-axis orbit of given energy
    double longaxisperiod(double E, double Xmax=-1) const;  // find period of oscillations for long-axis orbit of given energy
    double findintersection(double E, double X, double Y, double Z) const;       // finds k so that Phi(k*X, k*Y, k*Z) = E
};

class CPotentialLog: public CPotential
{
public:
    const double Rc;      // core radius of log potential
    CPotentialLog(int _N_dim, double _q, double _p, double _Mbh, double _Rc);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    virtual double Rho(double X, double Y, double Z, double t=0) const;
    virtual double Phi(double X, double Y, double Z, double t=0) const;
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const;
};

class CPotentialDehnen: public CPotential
{
public:
    const double Gamma;      // cusp exponent for Dehnen potential
    CPotentialDehnen(int _N_dim, double _q, double _p, double _Mbh, double _Gamma);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    virtual double Rho(double X, double Y, double Z, double t=0) const;
    virtual double Phi(double X, double Y, double Z, double t=0) const;
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const;
};

typedef double SHcoefsArray[MAX_NCOEFS_RADIAL][MAX_NCOEFS_ANGULAR][MAX_NCOEFS_ANGULAR];
class CPotentialBSE: public CPotential   // implement basis-set expansion on Zhao96 basis set (alpha models)
{
public:
    const double Gamma;      // cusp exponent for Dehnen potential (not used in potential model itself, only in its application to Schw grid
    const double Alpha;      // model parameter controlling inner and outer slopes
    CPotentialBSE(int _N_dim, double _q, double _p, double _Mbh, double _Gamma, double _Alpha, int _Ncoefs_radial, int _Ncoefs_angular, 
        int npoints=0, double X[]=NULL, double Y[]=NULL, double Z[]=NULL, double mass[]=NULL);
    CPotentialBSE(int _N_dim, double _q, double _p, double _Mbh, double _Gamma, double _Alpha, int _Ncoefs_radial, int _Ncoefs_angular, 
        SHcoefsArray _SHcoefs);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    virtual double Phi(double X, double Y, double Z, double t=0) const;
    virtual double Rho(double X, double Y, double Z, double t=0) const;  // density approximated by basis set
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const;
    int Ncoefs_radial, Ncoefs_angular;
    //int Ncoefs_angular_n[MAX_NCOEFS_RADIAL];  // to save time, number of angular coefficients used depends on the order 'n' of radial basis function
    mutable SHcoefsArray SHcoefs;
private:
    void prepareCoefs(int npoints, double X[], double Y[], double Z[], double mass[]);
};

class CPotentialDehnenBSE: public CPotentialBSE
{
public:
	CPotentialDehnenBSE(int _N_dim, double _q, double _p, double _Mbh, double _Gamma, int _Ncoefs_radial, int _Ncoefs_angular);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    double _Rho(double X, double Y, double Z, double t=0) const;         // original density in Dehnen model
#ifdef RHO_DEHNEN_EXACT
    virtual double Rho(double X, double Y, double Z, double t=0) const;
#endif
private:
    void prepareCoefs();
};

class CPotentialPlummerBSE: public CPotentialBSE
{
public:
	CPotentialPlummerBSE(int _N_dim, double _q, double _p, double _Mbh, int _Ncoefs_radial, int _Ncoefs_angular);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    double _Rho(double X, double Y, double Z, double t=0) const;         // original density in Plummer model
#ifdef RHO_DEHNEN_EXACT
    virtual double Rho(double X, double Y, double Z, double t=0) const;
#endif
private:
    void prepareCoefs();
};

class CPotentialScaleFree: public CPotential
{
public:
    double Gamma;      // scale-free profile exponent
    CPotentialScaleFree(int _N_dim, double _q, double _p, double _Mbh, double _Gamma);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const ;
    static char* myName();
    virtual double Rho(double X, double Y, double Z, double t=0) const;
    virtual double Phi(double X, double Y, double Z, double t=0) const;
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const;
};

class CPotentialScaleFreeBSE: public CPotentialScaleFree
{
public:
    CPotentialScaleFreeBSE(int _N_dim, double _q, double _p, double _Mbh, double _Gamma, int _Ncoefs_SF_angular);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    virtual double Phi(double X, double Y, double Z, double t=0) const;
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const;
	int Ncoefs_SF_angular;
    double SHcoefs[MAX_NCOEFS_SF_ANGULAR][MAX_NCOEFS_SF_ANGULAR];
    void prepareCoefs();
};

class CPotentialHarmonic: public CPotential 
{
public:
    CPotentialHarmonic(int _N_dim, double _q, double _p, double _Mbh);
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    virtual double Rho(double X, double Y, double Z, double t=0) const;
    virtual double Phi(double X, double Y, double Z, double t=0) const;
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const;
};

// frozen N-body potential calculated by tree-code algorithm (based on hackcode1.c)
class CPotentialNB: public CPotential   // frozen N-body potential
{
public:

enum NODETYPE{
    NODE,
    BODY,
    CELL
};

// NODE: data common to BODY and CELL structures.
typedef double vec3[N_DIM], matrix3[N_DIM][N_DIM];
struct node {
    NODETYPE type;                 /* code for node type */
    double mass;                  /* total mass of node */
    vec3 pos;			/* position of node */
};
typedef node *nodeptr;

// BODY: data structure used to represent particles.
typedef node body, *bodyptr;

// CELL: structure used to represent internal nodes of tree.
#define IMAX (1 << (8 * sizeof(int) - 2))       /* max integer coordinate */
#define NSUB (1 << N_DIM)        /* subcells per cell */
struct cell{
    NODETYPE type;
    double mass;                  /* total mass of cell */
    vec3 pos;                 /* cm. position of cell */
    nodeptr subp[NSUB];         /* descendents of cell */
#ifdef TREECODE_QUADRUPOLE
    matrix3 quad;		/* quad. moment of cell */
#endif
    cell() { type=CELL; for(int s=0; s<NSUB; s++) subp[s]=NULL; mass=-1;}
};
typedef cell *cellptr;

enum WALKOP{   // operation to perform during tree-walk: quantity to calculate
    CALCPHI,   // potential
    CALCACC   // acceleration(3 dim)
};  
public:
    const double Gamma;      // cusp exponent for Dehnen potential
    const double eps;         /* potential softening parameter */
    const double tol;         /* accuracy parameter: 0.0 => exact */
    
    CPotentialNB(int _N_dim, double _q, double _p, double _Mbh, double _Gamma, double _eps, double _tol, 
        int npoints=0, double X[]=NULL, double Y[]=NULL, double Z[]=NULL, double mass[]=NULL);
    CPotentialNB(int _N_dim, double _q, double _p, double _Mbh, double _Gamma, double _eps, double _tol, 
        int npoints, float X[], float Y[], float Z[], float mass[]);  // variant with float initialization
    virtual CPotential* clone() const;
    virtual POTENTIALTYPE PotentialType() const;
    virtual char* PotentialName() const;
    static char* myName();
    virtual double Phi(double X, double Y, double Z, double t=0) const;
    virtual double Rho(double X, double Y, double Z, double t=0) const;
    virtual void DiffEq(unsigned n, double t, double *v, double *f) const;
    mutable int nbb,nbc;
    std::vector<body> bodytab;
private:
    std::vector<cell> celltab;
    
    nodeptr troot;      /* tree root */
    vec3 rmin;          /* lower-left corner of coord. box */
    double rsize;       /* side-length of int. coord. box */
    // tree construction
    void maketree();                   // initialize tree for the constructor
    bool expandbox(const int b);             // expand bounding box (aka root cell) if particle does not fit into existing one
    bool appendtree(const int b);              // append particle to the tree 
    bool intcoord(int xp[N_DIM], const vec3 rp) const; // calculate integerized coords
    int subindex(const int x[N_DIM], const int l) const;     // determine subcell that a given position belongs to (l is cell level)
    void centerofmass(const nodeptr q);      // recursive calculation of cell center-of-mass
    // tree traversal
    void walktree(const nodeptr p, const double dsq, const vec3 pos0, const WALKOP operation, double data[]) const;
    void walktreedens(const nodeptr p, const double dsq, const double searchradsq, const vec3 pos0, const int xp[N_DIM], const int lev, vectorpd* data) const;
};

}