#pragma once
#include <vector>
#include <QString>
#include <QThread>
#include <fstream>
#include "orbit.h"
#include "potential.h"

namespace smile{

// Auxiliary class holding spatial grid for Schw.model
class CSpatialGrid
{
public:
    const int numShells;          // number of shells in model (both in coarse-graining configuration space and in start-space of orbits
    const int linesPerSegment;    // Schwarzschild's scheme for dividing octant into three segments, each of them divided into linesPerSegment^2 squares
    const int cellsPerSegment;
    const int cellsPerShell;
    const int cellsInModel;
    const CPotential* potential;
	vectord cellBoundary;   // angles dividing segment into linesPerSegment parts
    vectord shellBoundary;  // x-values of shell boundaries
    double totalMass;       // mass within finite grid shells

    CSpatialGrid(const CPotential* _potential, const int _numShells, const int _linesPerSegment);
    int whichCell(double X, double Y, double Z) const;                 // return index of cell containing given coordinates
    void cellCenter(int C, double& X, double& Y, double& Z) const;     // return coordinates of geometric center of a cell
    double potentialGamma() const;  // convenience function, returning Gamma value of potential (if it is not applicable, return -1)
};

// declare COrbitLibrary class defined in orbit.h
class COrbitLibrary;

class CSchwarzschildModel : public QObject
{
    Q_OBJECT
public:
    CSpatialGrid* grid;     // pointer to corresponding spatial grid
    COrbitLibrary* orbits;  // pointer to orbit library
    vectord shellEnergy;    // energy at corresponding boundary (at x-axis)
    vectord shellTorbx;     // unit time at each boundary (orbital period of x-axis orbit)
    vectord cellMass;       // required mass in each cell
    vectord cellPenalty;    // infeasible cells: difference between required and assigned mass in each cell after optimization

    int nOrbitsNonZero;              // orbits with non-zero weight
    double sumSquaredWeights;        // sum of squared orbit weights
    double massRegular, massChaotic; // mass in different components
    int nCellsInfeasible;            // number of cells whose mass cannot be built up from the superposition of orbits
    
    CSchwarzschildModel(const CPotential* _potential, const int _numShells, const int _linesPerSegment);
    ~CSchwarzschildModel();
    double solveOptimizationProblem(bool QP);           // solve linear or quadratic optimization problem
    double solveLucyIterationsOptimization(int &nIter); // solve optimization problem by Lucy iterations
    double calcCellPenalty();                           // find cells which are infeasible after optimization, return sum of deviations for these cells
    bool exportGrid(QString fileName);                  // export some statistics about grid and solution
    int exportNbodyPrepare(int numPoints, QString fileName, int refineFactor=0);    // check whether there are enough sampling points for all orbits to create Nbody model
    void exportNbody();                                 // export Schw.model to Nbody model
signals:
    void calcProgress(const QString& message);          // emitted on several occasions in the process of optimization or Nbody export
public slots:
    void stop();                          // abandon Nbody export if set externally
private:
    bool stopflag;                        // flag set on external event (stop current operation from GUI)
    std::vector<vectorf> linearMatrix;    // matrix M of linear equations for the optimization problem  "M w = rhs"
    vectorf rhs;                          // the rhs of equations
    vectorf constraintWeight;             // 'importance' of linear constraints (in the form of penalty for slack variables L, M)
    vectord orbitWeight;                  // the solution "w"
    vectord weightFactor;                 // penalties for individual orbits (=chaoticWeightFactor/numOrbits for chaotic orbits, 0 for regular ones)
    std::vector<int> numChaoticInShell;   // if using superorbits, keep the number of chaotic orbits in a superorbit at each shell
    int maxShellSuperOrbit;               // max number of shell for which all chaotic orbits will be averaged into super-orbit

    QString exportNbodyFileName;          // temporary storage variables to keep data from exportNbodyPrepare to actual exportNbody
    int     exportNbodyNumPoints;
    vectord exportNbodyOrbitMass;

    int doOptimizationProblem(bool QP);   // return 0 on success, <0 on error
    int doLucyIterationsOptimization();   // return nIter>0 on success, <0 on error
    void prepareModel();                  // prepare orbitCellWeight array before calling optimization routine
    void updateModel(bool finish=true);   // store orbitWeight after calling optimization routine; if finish=false, optimizations still continues (in Lucy iterations)
};

// helper class that writes NEMO-compatible snapshot file
class CSnapShotWriter
{
public:
   CSnapShotWriter(){level=0;};
   CSnapShotWriter(const QString name, bool append=false);
   ~CSnapShotWriter();
   void PutInt(const char*name,int i);
   void PutDouble(const char*name,double d);
   // write array of double; demi - number of dimensions, dem - length of array for each dimension
   void PutDoubleArray(const char*name, int demi, const int *dem, const double*);
   void PutFloatArray(const char*name, int demi, const int *dem, const float*);
   void PutIntArray(const char*name,int demi,const int *dem,const int*);
   void PutCharArray(const char*,int,const int*,const char*);
   void StartLevel(const char* name);
   void EndLevel();
   void PutString(const char*name,const char*str);
   void WriteHistory(QString new_h);
   void WritePhase(const std::vector<float> *v,
                   const std::vector<float> &m,
			       double t);
private:
   std::ofstream snap;
   int level;  //number of level (root 0)
   void PutZString(const char* name); 
};

}